from ninja import NinjaAPI

from django.core.cache import cache
from django.test import TestCase

from app_utils.testing import create_user_from_evecharacter

from killstats.api.killstats import KillboardApiEndpoints
from killstats.tests.test_api import _stats_api
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all

MODULE_PATH = "killstats.api.killstats.api_helper"


class Test_ApiStatsEndpoints(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_killstats_all()
        cache.clear()

        cls.user, _ = create_user_from_evecharacter(
            1001,
            permissions=["killstats.basic_access", "killstats.admin_access"],
        )

        cls.user2, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "killstats.basic_access",
            ],
        )
        cls.api = NinjaAPI()
        cls.manage_api_endpoints = KillboardApiEndpoints(api=cls.api)

    def test_top_ship_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/ship/top/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)

        # then
        expected_data = _stats_api.top_ship
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/ship/top/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)

        # then
        expected_data = _stats_api.top_ship
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/ship/top/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/ship/top/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_worst_ship_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/ship/worst/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.worst_ship
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/ship/worst/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.worst_ship
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/ship/worst/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/ship/worst/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_top_killer_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/killer/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_killer
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/top/killer/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_killer
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/top/killer/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/top/killer/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_victim_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/victim/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_victim
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_victim
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/top/victim/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_victim
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/top/victim/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_top_kill_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/kill/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_kill
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/top/kill/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_kill
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/top/kill/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/top/kill/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_top_loss_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/loss/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_loss
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/top/loss/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.top_loss
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/top/loss/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/top/loss/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_alltime_killer_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/alltime_killer/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.alltime_killer
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/top/alltime_killer/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.alltime_killer
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/top/alltime_killer/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/top/alltime_killer/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_alltime_victim_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/alltime_victim/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.alltime_victim
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Alliance
        url = "/killstats/api/stats/top/alltime_victim/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _stats_api.alltime_victim
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Empty Entry
        url = "/killstats/api/stats/top/alltime_victim/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Cached
        url = "/killstats/api/stats/top/alltime_victim/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = [{"stats": {}}]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_top_10_api(self):
        # given
        self.client.force_login(self.user)
        self.maxDiff = None

        # Corporation
        url = "/killstats/api/stats/top/10/month/7/year/2024/corporation/0/"
        # when
        response = self.client.get(url)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Top 10 Killers")

        # Alliance
        url = "/killstats/api/stats/top/10/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Top 10 Killers")

        # Empty Entry
        url = "/killstats/api/stats/top/10/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)

        # then
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Top 10 Killers")

        # Cached
        url = "/killstats/api/stats/top/10/month/7/year/1999/alliance/30000001/"
        # when
        response = self.client.get(url)

        # then
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Top 10 Killers")



top10 = [
    {"character_id": 1001, "kill_count": 3, "character_name": "Gneuten"},
    {"character_id": 1002, "kill_count": 3, "character_name": "rotze Rotineque"},
    {"character_id": 1004, "kill_count": 1, "character_name": "Garthd"},
    {"character_id": 1005, "kill_count": 1, "character_name": "Gerthd"},
    {"character_id": 1010, "kill_count": 1, "character_name": "Test1"},
    {"character_id": 1011, "kill_count": 1, "character_name": "Test2"},
    {"character_id": 1012, "kill_count": 1, "character_name": "Test3"},
    {"character_id": 1013, "kill_count": 1, "character_name": "Test4"},
]

top10_alt = [
    {"character_id": 1001, "kill_count": 3, "character_name": "Gneuten"},
    {"character_id": 1002, "kill_count": 3, "character_name": "rotze Rotineque"},
    {"character_id": 1004, "kill_count": 1, "character_name": "Garthd (Gneuten)"},
    {"character_id": 1005, "kill_count": 1, "character_name": "Gerthd"},
    {"character_id": 1010, "kill_count": 1, "character_name": "Test1"},
    {"character_id": 1011, "kill_count": 1, "character_name": "Test2"},
    {"character_id": 1012, "kill_count": 1, "character_name": "Test3"},
    {"character_id": 1013, "kill_count": 1, "character_name": "Test4"},
]

top10_unknown = [
    {"character_id": 1001, "kill_count": 3, "character_name": "Unknown"},
    {"character_id": 1002, "kill_count": 3, "character_name": "Unknown"},
    {"character_id": 1004, "kill_count": 1, "character_name": "Unknown"},
    {"character_id": 1005, "kill_count": 1, "character_name": "Unknown"},
    {"character_id": 1010, "kill_count": 1, "character_name": "Unknown"},
    {"character_id": 1011, "kill_count": 1, "character_name": "Unknown"},
    {"character_id": 1012, "kill_count": 1, "character_name": "Unknown"},
    {"character_id": 1013, "kill_count": 1, "character_name": "Unknown"},
]



# Standard Library
import calendar
from datetime import datetime

# Django
from django.template.defaulttags import register


@register.filter(name="current_month")
def current_month(value):
    try:
        month_number = int(value)
        return calendar.month_name[month_number]
    except (ValueError, TypeError, IndexError):
        return datetime.now().strftime("%B")



from unittest.mock import patch

from ninja import NinjaAPI

from django.core.cache import cache
from django.test import TestCase

from allianceauth.eveonline.models import EveAllianceInfo
from app_utils.testing import create_user_from_evecharacter

from killstats.api.killstats import KillboardApiEndpoints
from killstats.tests.test_api import _alliance_api
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all


class Test_AllianceEndpoints(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_killstats_all()
        cache.clear()

        cls.user, _ = create_user_from_evecharacter(
            1001,
            permissions=["killstats.basic_access", "killstats.admin_access"],
        )

        cls.user2, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "killstats.basic_access",
            ],
        )
        cls.api = NinjaAPI()
        cls.manage_api_endpoints = KillboardApiEndpoints(api=cls.api)

    def test_ally_killboard_api_own(self):
        # given
        self.client.force_login(self.user)

        # Halls of Fame/Shame - Main
        url = "/killstats/api/halls/month/7/year/2024/alliance/0/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _alliance_api.Killstats_Halls_Entry
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_ally_killmail_api_single(self):
        # given
        self.client.force_login(self.user)

        url = "/killstats/api/killmail/month/7/year/2024/alliance/30000001/kills/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _alliance_api.Killstats_Kills_Entry
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

        # Test Own Alliances
        url = "/killstats/api/killmail/month/7/year/2024/alliance/0/losses/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _alliance_api.Killstats_Losses_Entry
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_halls_api_single(self):
        # given
        self.client.force_login(self.user)

        # Halls of Fame/Shame - Single
        url = "/killstats/api/halls/month/7/year/2024/alliance/30000001/"
        # when
        response = self.client.get(url)
        # then
        expected_data = _alliance_api.Killstats_Halls_Single
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), expected_data)

    def test_get_alliance_admin(self):
        self.client.force_login(self.user2)
        url = "/killstats/api/killboard/alliance/admin/"
        # when
        response = self.client.get(url)
        # then
        excepted_data = [
            {
                "alliance": {
                    "30000001": {
                        "alliance_id": 30000001,
                        "alliance_name": "Voices of War",
                    }
                }
            }
        ]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), excepted_data)

    @patch("killstats.api.killstats.admin.AlliancesAudit.objects.visible_to")
    def test_get_alliance_admin_no_visible(self, mock_visible_to):
        self.client.force_login(self.user2)
        url = "/killstats/api/killboard/alliance/admin/"

        mock_visible_to.return_value = None

        # when
        response = self.client.get(url)
        # then
        self.assertContains(response, "Permission Denied", status_code=403)

    @patch("killstats.api.killstats.admin.AlliancesAudit.objects.visible_to")
    def test_get_alliance_admin_exception(self, mock_visible_to):
        self.client.force_login(self.user)
        url = "/killstats/api/killboard/alliance/admin/"

        corp = EveAllianceInfo.objects.get(alliance_id=30000001)

        mock_visible_to.return_value = [corp, "test"]

        # when
        response = self.client.get(url)
        # then
        self.assertEqual(response.status_code, 200)



from http import HTTPStatus
from unittest.mock import Mock, patch

from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse
from esi.models import Token

from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)
from app_utils.testing import create_user_from_evecharacter

from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.views import (
    add_alliance,
    add_corp,
    alliance_admin,
    corporation_admin,
    killboard_index,
)

MODULE_PATH = "killstats.views"


class KillstatsAuditTest(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.basic_access",
                "killstats.admin_access",
            ],
        )

    def test_view(self):
        request = self.factory.get(reverse("killstats:index"))
        request.user = self.user
        response = killboard_index(request, 0)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_view_single(self):
        request = self.factory.get(reverse("killstats:index"))
        request.user = self.user
        response = killboard_index(request, 20000001)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_view_corporation_admin(self):
        request = self.factory.get(reverse("killstats:corporation_admin"))
        request.user = self.user
        response = corporation_admin(request)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_view_alliance_admin(self):
        request = self.factory.get(reverse("killstats:alliance_admin"))
        request.user = self.user
        response = alliance_admin(request)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".killmail_update_corp")
    def test_add_corp(self, mock_update_corp, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker
        request = self.factory.get(reverse("killstats:add_corp"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_corp.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.info.called)
        self.assertTrue(mock_update_corp.apply_async.called)
        self.assertTrue(
            CorporationsAudit.objects.get(
                corporation=self.character_ownership.character.corporation
            )
        )

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".killmail_update_corp")
    def test_add_corp_not_created(self, mock_update_corp, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker

        corp, _ = EveCorporationInfo.objects.get_or_create(
            corporation_id=token.corporation_id,
            defaults={
                "member_count": 0,
                "corporation_ticker": token.corporation_ticker,
                "corporation_name": token.corporation_name,
            },
        )
        CorporationsAudit.objects.update_or_create(
            corporation=corp, owner=self.character_ownership.character
        )

        request = self.factory.get(reverse("killstats:add_corp"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_corp.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.info.called)
        self.assertFalse(mock_update_corp.apply_async.called)
        self.assertTrue(
            CorporationsAudit.objects.get(
                corporation=self.character_ownership.character.corporation
            )
        )

    @patch(MODULE_PATH + ".messages")
    def test_add_corp_char_not_found(self, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = 999999
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker

        request = self.factory.get(reverse("killstats:add_corp"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_corp.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        mock_messages.error.assert_called_once()

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".EveCharacter.objects.get_character_by_id")
    def test_npc_corporation(self, mock_char, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = 999_999

        mock_char.return_value = self.character_ownership.character
        mock_char.return_value.corporation_id = 999_999

        request = self.factory.get(reverse("killstats:add_corp"))
        request.user = self.user
        request.token = token

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_corp.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.error.called)


class KillstatsAllianceAuditTest(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.basic_access",
                "killstats.admin_access",
            ],
        )

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".killmail_update_corp")
    def test_add_alliance(self, mock_update_corp, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker
        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_alliance.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.info.called)
        # self.assertTrue(mock_update_corp.apply_async.called)
        self.assertTrue(
            AlliancesAudit.objects.get(
                alliance=self.character_ownership.character.alliance
            )
        )

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".killmail_update_corp")
    @patch(MODULE_PATH + ".EveAllianceInfo.objects.get")
    def test_add_alliance_none(self, mock_alliance, mock_update_corp, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker
        mock_alliance.return_value = None

        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_alliance.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.error.called)

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".EveCharacter.objects.get_character_by_id")
    def test_no_alliance(self, mock_char, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id

        mock_char.return_value = self.character_ownership.character
        mock_char.return_value.alliance_id = None

        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = self.user
        request.token = token

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_alliance.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.error.called)

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".killmail_update_corp")
    @patch(MODULE_PATH + ".EveAllianceInfo.objects.create_alliance")
    @patch(MODULE_PATH + ".EveAllianceInfo.objects.get")
    def test_add_alliance_not_exist(
        self, mock_alliance, mock_alliance_create, mock_update_corp, mock_messages
    ):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker

        mock_alliance.side_effect = EveAllianceInfo.DoesNotExist
        mock_alliance_create.return_value = EveAllianceInfo(
            alliance_id=3999,
            alliance_name="Test Corp",
            alliance_ticker="TC",
            executor_corp_id=2999,
        )
        mock_alliance_create.return_value.save()

        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_alliance.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.info.called)

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".killmail_update_corp")
    def test_add_alliance_not_created(self, mock_update_corp, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = self.character_ownership.character.character_id
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker

        ally = EveAllianceInfo.objects.get(
            alliance_id=30000001,
        )
        AlliancesAudit.objects.update_or_create(
            alliance=ally, owner=self.character_ownership.character
        )

        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_alliance.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        self.assertTrue(mock_messages.info.called)
        self.assertFalse(mock_update_corp.apply_async.called)
        self.assertTrue(
            AlliancesAudit.objects.get(
                alliance=self.character_ownership.character.alliance
            )
        )

    @patch(MODULE_PATH + ".messages")
    def test_add_alliance_char_not_found(self, mock_messages):
        self.client.force_login(self.user)
        token = Mock(spec=Token)
        token.character_id = 999999
        token.corporation_id = self.character_ownership.character.corporation_id
        token.corporation_name = self.character_ownership.character.corporation_name
        token.corporation_ticker = self.character_ownership.character.corporation_ticker

        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = self.user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # given
        orig_view = add_alliance.__wrapped__.__wrapped__
        # when
        response = orig_view(request, token)
        # then
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("killstats:index"))
        mock_messages.error.assert_called_once()



from unittest.mock import patch

from django.core.exceptions import ObjectDoesNotExist
from django.test import RequestFactory, TestCase
from django.urls import reverse

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from app_utils.testing import create_user_from_evecharacter

from killstats.api.account_manager import AccountManager
from killstats.api.killstats.api_helper import get_alliances, get_corporations
from killstats.tests.testdata.load_allianceauth import load_allianceauth

MODULE_PATH = "killstats.api.account_manager"


class TestApiHelpers(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        cls.factory = RequestFactory()

        cls.user, cls.char_owner = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.basic_access",
            ],
        )
        cls.user2, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "killstats.basic_access",
            ],
        )
        cls.user3, _ = create_user_from_evecharacter(
            1003,
        )
        cls.user4, _ = create_user_from_evecharacter(
            1004,
        )
        cls.user5, _ = create_user_from_evecharacter(
            1005,
        )
        cls.corp = EveCorporationInfo.objects.get(corporation_id=20000001)

    def test_get_main_and_alts_all_char_in_chars(self):
        # given
        mains = {}
        request = self.factory.get("/")
        request.user = self.user
        chars = EveCharacter.objects.filter(
            corporation_id__in=[self.corp.corporation_id]
        )
        for char in chars:
            mains[char.character_id] = {"main": char, "alts": [char]}
        excepted_data = mains
        account = AccountManager(entities=[self.corp.corporation_id])
        # when
        data, _ = account.get_mains_alts()
        # then
        self.assertEqual(data, excepted_data)

    @patch(MODULE_PATH + ".EveCharacter.objects.select_related")
    def test_get_main_and_alts_all_process_object_does_not_exist(
        self, mock_select_related
    ):
        # Setup mock EveCharacter instance to simulate ObjectDoesNotExist
        mock_select_related.return_value.get.side_effect = ObjectDoesNotExist

        # Setup the rest of your test environment as before
        mains = {}
        request = self.factory.get("/")
        request.user = self.user
        chars = EveCharacter.objects.filter(
            corporation_id__in=[self.corp.corporation_id]
        )
        for char in chars:
            mains[char.character_id] = {"main": char, "alts": [char]}
        account = AccountManager(entities=[self.corp.corporation_id])
        # when
        data, _ = account.get_mains_alts()

    def test_process_character(self):
        # Create mock main character
        main_char = EveCharacter(character_id=1002, corporation_id=20000002)

        # Mock char with a main character
        char = EveCharacter(character_id=1001, corporation_id=20000001)
        char.character_ownership = self.char_owner
        char.character_ownership.user.profile.main_character = main_char

        # Test main not in characters
        characters = {}
        chars_list = set()

        account = AccountManager(entities=[20000001])

        account._process_character(char, characters, chars_list)

        # Test main in characters
        characters = {1002: {"main": main_char, "alts": []}}
        chars_list = set()

        account._process_character(char, characters, chars_list)

        # Test Corporation exist
        characters = {}
        chars_list = set()

        account._process_character(
            char,
            characters,
            chars_list,
        )

        # Test Corporation not exist
        char = EveCharacter(character_id=1001, corporation_id=20000001)
        characters = {}
        chars_list = set()

        account._process_character(char, characters, chars_list)

        # Test Attribute Error
        error = EveCorporationInfo.objects.get(corporation_id=20000001)
        characters = {}
        chars_list = set()

        account._process_character(error, characters, chars_list)

        # Test Missing character
        char = EveCharacter(character_id=9999, corporation_id=20000001)
        characters = {}
        chars_list = set()

        account._process_character(char, characters, chars_list)

        # Test Corporation & Alliance Empty
        account = AccountManager()
        char = EveCharacter(character_id=1001, corporation_id=20000001)
        characters = {}
        chars_list = set()

        account._process_character(char, characters, chars_list)

    def test_no_corp_or_ally(self):

        request = self.factory.get(reverse("killstats:index"))
        request.user = self.user

        corp_list = get_corporations(request)

        expected_data = []

        self.assertEqual(corp_list, expected_data)

        ally_list = get_alliances(request)

        expected_data = []

        self.assertEqual(ally_list, expected_data)



import json
from unittest.mock import MagicMock, call, patch

from django.db import IntegrityError
from django.test import TestCase

from killstats.managers.killmail_core import KillmailManager
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit
from killstats.tasks import (
    killmail_fetch_all,
    killmail_update_ally,
    killmail_update_corp,
    store_killmail,
)
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import (
    _load_get_bulk_data,
    load_killstats_all,
)

MODULE_PATH = "killstats.tasks"


def load_test_data(filename):
    with open(filename) as file:
        return json.load(file)


class TestTasks(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        load_killstats_all()

    @patch(MODULE_PATH + ".killmail_update_corp.apply_async")
    @patch(MODULE_PATH + ".killmail_update_ally.apply_async")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_fetch_all(self, mock_logger, mock_update_ally, mock_update_corp):
        # given
        corp_count = CorporationsAudit.objects.count()
        ally_count = AlliancesAudit.objects.count()
        expected_count = corp_count + ally_count
        # when
        killmail_fetch_all()
        # then
        self.assertEqual(mock_update_corp.call_count, corp_count)
        self.assertEqual(mock_update_ally.call_count, ally_count)
        mock_logger.info.assert_called_once_with(
            "Queued %s Killstats Audit Updates", expected_count
        )

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_update_corp(self, mock_logger, mock_get_killmail_data_bulk):
        # given
        corp = CorporationsAudit.objects.get(corporation__corporation_id=20000001)

        mock_get_killmail_data_bulk.return_value = _load_get_bulk_data()
        # when
        killmail_update_corp(corp.corporation.corporation_id)
        # then
        mock_logger.info.assert_called_once_with(
            "Killboard runs completed. %s killmails received from zKB for %s",
            3,
            corp.corporation.corporation_name,
        )

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_update_ally(self, mock_logger, mock_get_killmail_data_bulk):
        # given
        ally = AlliancesAudit.objects.get(alliance__alliance_id=30000001)

        mock_get_killmail_data_bulk.return_value = _load_get_bulk_data()
        # when
        killmail_update_ally(ally.alliance.alliance_id)
        # then
        mock_logger.info.assert_called_once_with(
            "Killboard runs completed. %s killmails received from zKB for %s",
            3,
            ally.alliance.alliance_name,
        )

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_update_corp_no_new(
        self, mock_logger, mock_get_killmail_data_bulk
    ):
        # given
        corp = CorporationsAudit.objects.get(corporation__corporation_id=20000001)

        mock_get_killmail_data_bulk.return_value = None
        # when
        killmail_update_corp(corp.corporation.corporation_id)
        # then
        mock_logger.debug.assert_called_once_with("No new Killmail found.")

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_update_ally_no_new(
        self, mock_logger, mock_get_killmail_data_bulk
    ):
        # given
        ally = AlliancesAudit.objects.get(alliance__alliance_id=30000001)

        mock_get_killmail_data_bulk.return_value = None
        # when
        killmail_update_ally(ally.alliance.alliance_id)
        # then
        mock_logger.debug.assert_called_once_with("No new Killmail found.")

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_store(self, mock_logger, mock_get_killmail_data_bulk):
        # given
        killmail_list = _load_get_bulk_data()
        for killmail_dict in killmail_list:
            killmail = KillmailManager._create_from_dict(killmail_dict)
            killmail.save()
        # when
        store_killmail(killmail.id)
        # then
        mock_logger.debug.assert_called_once_with("%s: Stored killmail", 119271433)

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".Killmail.objects.create_from_killmail")
    @patch(MODULE_PATH + ".KillmailManager.get")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_store_and_already_exist(
        self,
        mock_logger,
        mock_get,
        mock_create_from_killmail,
        mock_get_killmail_data_bulk,
    ):
        # given
        killmail_id = 119271433
        killmail_mock = MagicMock(id=killmail_id)
        mock_get.return_value = killmail_mock
        mock_create_from_killmail.side_effect = IntegrityError
        # when
        store_killmail(killmail_id)
        # then
        mock_logger.warning.assert_called_once_with(
            "%s: Failed to store killmail, because it already exists", killmail_id
        )

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".Killmail.objects.all")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_update_corp_existing_kms(
        self, mock_logger, mock_existing, mock_get_killmail_data_bulk
    ):
        # given
        corp = CorporationsAudit.objects.get(corporation__corporation_id=20000001)
        existing_kms = [119324952, 119324561, 119271433]
        mock_get_killmail_data_bulk.return_value = _load_get_bulk_data()
        mock_existing.return_value.values_list.return_value = existing_kms
        # when
        killmail_update_corp(corp.corporation.corporation_id)
        # then
        mock_logger.debug.assert_called_with("No new Killmail found.")

    @patch(MODULE_PATH + ".KillmailManager.get_killmail_data_bulk")
    @patch(MODULE_PATH + ".Killmail.objects.all")
    @patch(MODULE_PATH + ".logger")
    def test_killmail_update_ally_existing_kms(
        self, mock_logger, mock_existing, mock_get_killmail_data_bulk
    ):
        # given
        ally = AlliancesAudit.objects.get(alliance__alliance_id=30000001)
        existing_kms = [119324952, 119324561, 119271433]
        mock_get_killmail_data_bulk.return_value = _load_get_bulk_data()
        mock_existing.return_value.values_list.return_value = existing_kms
        # when
        killmail_update_ally(ally.alliance.alliance_id)
        # then
        mock_logger.debug.assert_called_with("No new Killmail found.")



import unittest
from unittest.mock import MagicMock, patch

from django.core.exceptions import ObjectDoesNotExist

from killstats.api.account_manager import AccountManager

MODULE_PATH = "killstats.api.account_manager"


class Test_AccountManager(unittest.TestCase):
    def setUp(self):
        self.account_manager = AccountManager()
        self.account_manager.entities = {
            20000001,
            20000002,
            20000003,
            30000001,
            30000002,
            30000003,
        }

    @patch(MODULE_PATH + ".EveCharacter")
    def test_process_character_in_corporation(self, MockEveCharacter):
        char = MockEveCharacter()
        char.corporation_id = 20000001
        char.alliance_id = 1
        char.character_ownership.user.profile.main_character = char
        characters = {}
        chars_list = set()

        self.account_manager._process_character(char, characters, chars_list)

        self.assertIn(char.character_id, chars_list)
        self.assertIn(char, characters[char.character_id]["alts"])

    @patch(MODULE_PATH + ".EveCharacter")
    def test_process_character_in_alliance(self, MockEveCharacter):
        char = MockEveCharacter()
        char.corporation_id = 1
        char.alliance_id = 30000001
        char.character_ownership.user.profile.main_character = char
        characters = {}
        chars_list = set()

        self.account_manager._process_character(char, characters, chars_list)

        self.assertIn(char.character_id, chars_list)
        self.assertIn(char, characters[char.character_id]["alts"])

    @patch(MODULE_PATH + ".EveCharacter")
    def test_process_character_not_in_entities(self, MockEveCharacter):
        char = MockEveCharacter()
        char.corporation_id = 100
        char.alliance_id = 10
        char.character_ownership.user.profile.main_character = char
        characters = {}
        chars_list = set()

        self.account_manager._process_character(char, characters, chars_list)

        self.assertIn(char.character_id, characters)
        self.assertNotIn(char, characters[char.character_id]["alts"])
        self.assertNotIn(char.character_id, chars_list)



from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q

from allianceauth.eveonline.models import EveCharacter

from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class AccountManager:
    def __init__(self, entities=None) -> None:
        self.entities = entities if entities is not None else []

    def _get_linked_characters(self):
        query_filter = Q(corporation_id__in=self.entities)
        query_filter |= Q(
            character_ownership__user__profile__main_character__corporation_id__in=self.entities
        )
        query_filter |= Q(alliance_id__in=self.entities)
        query_filter |= Q(
            character_ownership__user__profile__main_character__alliance_id__in=self.entities
        )

        linked_chars = EveCharacter.objects.filter(query_filter)
        return (
            linked_chars.select_related(
                "character_ownership",
                "character_ownership__user__profile__main_character",
            )
            .prefetch_related("character_ownership__user__character_ownerships")
            .order_by("character_name")
        )

    def _process_character(self, char: EveCharacter, characters: dict, chars_list: set):
        try:
            main = char.character_ownership.user.profile.main_character
            if main and main.character_id not in characters:
                characters[main.character_id] = {"main": main, "alts": []}
            if (
                char.corporation_id in self.entities
                or char.alliance_id in self.entities
            ):
                characters[main.character_id]["alts"].append(char)
                chars_list.add(char.character_id)
        except ObjectDoesNotExist:
            if EveCharacter.objects.filter(character_id=char.character_id).exists():
                char = EveCharacter.objects.get(character_id=char.character_id)
                characters[char.character_id] = {"main": char, "alts": []}
                if (
                    char.corporation_id in self.entities
                    or char.alliance_id in self.entities
                ):
                    chars_list.add(char.character_id)
                    characters[char.character_id]["alts"].append(char)
        except AttributeError:
            pass

    def get_mains_alts(self):
        """Get all members for given corporations/alliances"""
        characters = {}
        chars_list = set()

        linked_chars = self._get_linked_characters()

        for char in linked_chars:
            self._process_character(char, characters, chars_list)
        return characters, list(chars_list)



from django.db import models

# Alliance Auth (External Libs)
from allianceauth.eveonline.evelinks import eveimageserver

# AA Killstats
from killstats.hooks import get_extension_logger
from killstats.models.killboard import Attacker, Killmail

logger = get_extension_logger(__name__)


def _get_character_details_attacker(killmail: Killmail, mains, entities, unique_killer):
    def process_attacker(attacker, mains, unique_killer):
        for mains_entry in mains.values():
            main_character = mains_entry["main"]
            alts = mains_entry["alts"]
            for alt_character in alts:
                if attacker.character_id == alt_character.character_id:
                    if attacker.character_id not in unique_killer:
                        main = main_character
                        alt = (
                            alt_character
                            if main_character.character_id != attacker.character_id
                            else None
                        )
                        character_id = (
                            main.character_id if alt is None else alt.character_id
                        )
                        character_name = (
                            f"{main.character_name}"
                            if alt is None
                            else f"{alt.character_name} ({main.character_name})"
                        )
                        unique_killer.add(attacker.character_id)
                        return character_id, character_name
        return None, None

    attackers = Attacker.objects.filter(
        models.Q(corporation_id__in=entities)
        | models.Q(alliance_id__in=entities)
        | models.Q(character_id__in=entities),
        killmail=killmail,
    )

    for attacker in attackers:
        character_id, character_name = process_attacker(attacker, mains, unique_killer)
        if character_id is not None:
            return character_id, character_name

    return None, None


def _get_character_details_victim(killmail: Killmail, mains):
    def process_victim(killmail, mains_entry):
        main_character = mains_entry["main"]
        alts = mains_entry["alts"]
        for alt_character in alts:
            if killmail.victim.id == alt_character.character_id:
                main = main_character
                alt = (
                    alt_character
                    if main_character.character_id != killmail.victim.id
                    else None
                )
                character_id = main.character_id if alt is None else alt.character_id
                character_name = (
                    f"{main.character_name}"
                    if alt is None
                    else f"{alt.character_name} ({main.character_name})"
                )
                return character_id, character_name
        return None, None

    try:
        for mains_entry in mains.values():
            character_id, character_name = process_victim(killmail, mains_entry)
            if character_id is not None:
                return character_id, character_name
    except AttributeError:
        pass

    return killmail.victim.id, killmail.victim.name


# pylint: disable=too-many-positional-arguments
def _hall_killmail(
    kms,
    stats: list,
    mains,
    entities=None,
    title=None,
    mode="killmail",
):
    unique_killer = set()
    for killmail in kms:
        if len(stats) >= 5:
            break

        zkb_link = killmail.evaluate_zkb_link()
        title_html = (
            f"{killmail.get_or_unknown_victim_name()}" if title is None else title
        )

        character_id, character_name = (
            _get_character_details_attacker(killmail, mains, entities, unique_killer)
            if mode == "attacker"
            else _get_character_details_victim(killmail, mains)
        )

        if character_id is None:
            continue

        portrait = eveimageserver.character_portrait_url(character_id, 256)

        if mode == "attacker":
            zkb_link = f"https://zkillboard.com/character/{character_id}/"

        stats.append(
            {
                # zKB Data
                "killmail_id": killmail.killmail_id,
                "character_id": character_id,
                "character_name": character_name,
                "ship": killmail.victim_ship_id,
                "ship_name": killmail.get_or_unknown_victim_ship_name(),
                "totalValue": killmail.victim_total_value,
                # Additional Data
                "title": title_html,
                "portrait": portrait,
                "zkb_link": zkb_link,
            }
        )
    return stats


# pylint: disable=too-many-locals
def killboard_hall(killmail_month: Killmail, entities, mains):
    # Ensure that the killmails are sorted by total value
    killmail_month = killmail_month.order_by("-victim_total_value").prefetch_related(
        "victim", "victim_ship"
    )

    shame = []
    fame = []

    topkiller = killmail_month.filter_entities_kills(entities)
    toplosses = killmail_month.filter_entities_losses(entities)

    _hall_killmail(
        topkiller, stats=fame, mains=mains, mode="attacker", entities=entities
    )

    _hall_killmail(toplosses, stats=shame, mains=mains)

    return shame, fame


def format_killmail(killmail: Killmail, title):
    return {
        "killmail_id": killmail.killmail_id,
        "character_id": killmail.victim_id,
        "character_name": killmail.get_or_unknown_victim_name(),
        "corporation_id": killmail.victim_corporation_id,
        "alliance_id": killmail.victim_alliance_id,
        "ship": killmail.victim_ship_id,
        "ship_name": killmail.get_or_unknown_victim_ship_name(),
        "hash": killmail.hash,
        "totalValue": killmail.victim_total_value,
        "date": killmail.killmail_date.isoformat(),
        "title": title,
        "portrait": f"https://images.evetech.net/types/{killmail.victim_ship.id}/icon?size=256",
        "zkb_link": f"https://zkillboard.com/kill/{killmail.killmail_id}/",
    }


def format_killmail_details(
    entity, loss=False, title=None, count=0, stats_type="character"
):
    if stats_type == "ship":
        return {
            "title": title,
            "type": "count",
            "ship_id": entity.id,
            "ship_name": entity.name,
            "portrait": f"https://images.evetech.net/types/{entity.id}/icon?size=256",
            "count": count,
            "loss": loss,
            "zkb_link": f"https://zkillboard.com/ship/{entity.id}/",
        }

    return {
        "title": f"{title} {entity.name}",
        "type": "count",
        "character_id": entity.id,
        "character_name": entity.name if entity.name is not None else "Unknown",
        "portrait": f"https://images.evetech.net/characters/{entity.id}/portrait?size=256",
        "count": count,
        "loss": loss,
        "zkb_link": f"https://zkillboard.com/character/{entity.id}/",
    }



from unittest.mock import MagicMock, patch

from django.test import RequestFactory, TestCase
from django.urls import reverse
from eveuniverse.models import EveEntity

from allianceauth.eveonline.models import EveCharacter
from app_utils.testing import add_character_to_user, create_user_from_evecharacter

from killstats.api.killstats.api_helper import cache_sytem, get_top_10, set_cache_key
from killstats.tests.test_api import _api_helper
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all

MODULE_PATH = "killstats.api.killstats.api_helper"


class Test_ApiHelper(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_killstats_all()
        cls.request = RequestFactory()

        cls.user, _ = create_user_from_evecharacter(
            1001,
            permissions=["killstats.basic_access", "killstats.admin_access"],
        )

        cls.user2, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "killstats.basic_access",
            ],
        )

    def test_top10(self):
        request = self.request.get(reverse("killstats:index"))
        request.user = self.user
        response = get_top_10(
            request, month=7, year=2024, entity_type="corporation", entity_id=20000001
        )

        expected_data = _api_helper.top10

        self.assertEqual(response, expected_data)

    def test_top10_alt(self):
        add_character_to_user(self.user, EveCharacter.objects.get(character_id=1004))
        request = self.request.get(reverse("killstats:index"))
        request.user = self.user
        response = get_top_10(
            request, month=7, year=2024, entity_type="alliance", entity_id=30000001
        )

        expected_data = _api_helper.top10_alt

        self.assertEqual(response, expected_data)

    def test_top10_empty(self):
        request = self.request.get(reverse("killstats:index"))
        request.user = self.user
        response = get_top_10(
            request, month=7, year=1999, entity_type="alliance", entity_id=30000001
        )

        expected_data = {}

        self.assertEqual(response, expected_data)

    @patch(MODULE_PATH + ".EveEntity.objects.get")
    def test_get_top_10_does_not_exist(self, mock_get):
        mock_get.side_effect = EveEntity.DoesNotExist

        request = self.request.get(reverse("killstats:index"))
        request.user = self.user

        expected_data = _api_helper.top10_unknown

        response = get_top_10(
            request, month=7, year=2024, entity_type="corporation", entity_id=20000001
        )
        self.assertEqual(response, expected_data)

    @patch(MODULE_PATH + ".EveEntity.objects.get")
    def test_get_top_10_attribute_error(self, mock_get):
        mock_get.side_effect = AttributeError

        request = self.request.get(reverse("killstats:index"))
        request.user = self.user

        result = get_top_10(
            request, month=7, year=2024, entity_type="corporation", entity_id=20000001
        )

        expected_data = _api_helper.top10_unknown

        self.assertEqual(result, expected_data)

    def test_cache_system(self):
        request = self.request.get(reverse("killstats:index"))
        request.user = self.user

        cache_key_name = "test_cache"

        # Test No Cache
        cache, cache_key = cache_sytem(request, cache_key_name, entity_id=20000001)

        expected_data = "test_cache_20000001"

        self.assertEqual(cache_key, expected_data)
        self.assertIsNone(cache)

        # Test Cache
        set_cache_key(cache_key, "test")

        cache, cache_key = cache_sytem(request, cache_key_name, entity_id=20000001)

        self.assertIsNone(cache_key)
        self.assertEqual(cache, "test")

        # Test Cache Hashed
        set_cache_key(cache_key, "test")

        cache, cache_key = cache_sytem(request, cache_key_name, entity_id=0)

        self.assertEqual(cache_key, "test_cache_f8183f422d05dda8d390178394e2b670")
        self.assertIsNone(cache)

    def test_set_cache_key_false(self):
        request = self.request.get(reverse("killstats:index"))
        request.user = self.user

        result = set_cache_key(None, "test")
        self.assertFalse(result)



from ninja import NinjaAPI
from ninja.security import django_auth

from django.conf import settings

from killstats.api import killstats
from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)

api = NinjaAPI(
    title="Killstats API",
    version="0.2.0",
    urls_namespace="killstats:new_api",
    auth=django_auth,
    csrf=True,
    openapi_url=settings.DEBUG and "/openapi.json" or "",
)

# Add the character endpoints
killstats.setup(api)



"""App Tasks"""

from celery import chain as Chain
from celery import shared_task

# Django
from django.db import IntegrityError
from django.utils import timezone

from allianceauth.services.tasks import QueueOnce

# AA Killstatsp
from killstats import app_settings
from killstats.decorators import when_esi_is_available
from killstats.hooks import get_extension_logger
from killstats.managers.killboard_manager import KillmailManager
from killstats.models.killboard import Killmail
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit

logger = get_extension_logger(__name__)


@shared_task
@when_esi_is_available
def killmail_fetch_all(runs: int = 0):
    corps_query = CorporationsAudit.objects.all()
    allys_query = AlliancesAudit.objects.all()
    for corp in corps_query:
        killmail_update_corp.apply_async(args=[corp.corporation.corporation_id])
        runs = runs + 1
    for ally in allys_query:
        killmail_update_ally.apply_async(args=[ally.alliance.alliance_id])
        runs = runs + 1
    logger.info("Queued %s Killstats Audit Updates", runs)


# pylint: disable=unused-argument
@shared_task(bind=True, base=QueueOnce)
@when_esi_is_available
def killmail_update_corp(self, corp_id: int, runs: int = 0):
    """Updates the killmails for a corporation"""
    corp = CorporationsAudit.objects.get(corporation__corporation_id=corp_id)
    killstats_url = f"https://zkillboard.com/api/npc/0/corporationID/{corp.corporation.corporation_id}/"

    killmail_list = KillmailManager.get_killmail_data_bulk(killstats_url)
    existing_kms = Killmail.objects.all().values_list("killmail_id", flat=True)
    if killmail_list:
        for killmail_dict in killmail_list:
            killmail = KillmailManager._create_from_dict(killmail_dict)
            killmail.save()
            if killmail.id in existing_kms:
                continue
            runs = runs + (1 if killmail else 0)
            Chain(
                store_killmail.si(killmail.id),
            ).delay()
        corp.last_update = timezone.now()
        corp.save()

    if not runs == 0:
        logger.info(
            "Killboard runs completed. %s killmails received from zKB for %s",
            runs,
            corp.corporation.corporation_name,
        )
    else:
        logger.debug("No new Killmail found.")


# pylint: disable=unused-argument
@shared_task(bind=True, base=QueueOnce)
@when_esi_is_available
def killmail_update_ally(self, alliance_id: int, runs: int = 0):
    """Updates the killmails for a corporation"""
    ally = AlliancesAudit.objects.get(alliance__alliance_id=alliance_id)
    killstats_url = (
        f"https://zkillboard.com/api/npc/0/allianceID/{ally.alliance.alliance_id}/"
    )

    killmail_list = KillmailManager.get_killmail_data_bulk(killstats_url)

    existing_kms = Killmail.objects.all().values_list("killmail_id", flat=True)

    if killmail_list:
        for killmail_dict in killmail_list:
            killmail = KillmailManager._create_from_dict(killmail_dict)
            killmail.save()
            if killmail.id in existing_kms:
                continue
            runs = runs + (1 if killmail else 0)
            Chain(
                store_killmail.si(killmail.id),
            ).delay()
        ally.last_update = timezone.now()
        ally.save()

    if not runs == 0:
        logger.info(
            "Killboard runs completed. %s killmails received from zKB for %s",
            runs,
            ally.alliance.alliance_name,
        )
    else:
        logger.debug("No new Killmail found.")


@shared_task(timeout=app_settings.KILLBOARD_TASKS_TIMEOUT)
def store_killmail(killmail_id: int) -> None:
    """stores killmail as EveKillmail object"""
    killmail = KillmailManager.get(killmail_id)
    try:
        Killmail.objects.create_from_killmail(killmail)
    except IntegrityError:
        logger.warning(
            "%s: Failed to store killmail, because it already exists", killmail.id
        )
    else:
        logger.debug("%s: Stored killmail", killmail.id)



from allianceauth.eveonline.models import EveCharacter

from killstats import models
from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


# NOTE: not implemented yet
def get_permission(request, entity_type: str):  # pragma: no cover
    """Get permission for the entity"""
    if entity_type == "corporation":
        entities = get_corporations(request)
    else:
        entities = get_alliances(request)

    if not entities:
        output = [{"No Data": "No data available for the entity"}]
        return False, output
    return True, entities


def get_corporations(request):
    linked_characters = request.user.profile.main_character.character_ownership.user.character_ownerships.select_related(
        "character", "user"
    ).all()

    linked_characters = linked_characters.values_list("character_id", flat=True)
    chars = EveCharacter.objects.filter(id__in=linked_characters)

    # Get all corporations and ensure they are not NPC corporations
    corporations = (
        chars.filter(corporation_id__gt=10_000_000)
        .values_list("corporation_id", flat=True)
        .distinct()
    )

    main_corp = models.CorporationsAudit.objects.filter(
        corporation__corporation_id__in=corporations
    )

    # Check access
    visible = models.CorporationsAudit.objects.visible_to(request.user)

    common_corps = main_corp.intersection(visible)
    if not common_corps.exists():
        return []

    return list(corporations)


def get_alliances(request):
    linked_characters = request.user.profile.main_character.character_ownership.user.character_ownerships.select_related(
        "character", "user"
    ).all()

    linked_characters = linked_characters.values_list("character_id", flat=True)
    chars = EveCharacter.objects.filter(id__in=linked_characters)

    alliances = chars.values_list("alliance_id", flat=True).distinct()

    main_corp = models.AlliancesAudit.objects.filter(
        alliance__alliance_id__in=alliances
    )

    # Check access
    visible = models.AlliancesAudit.objects.visible_to(request.user)

    common_corps = main_corp.intersection(visible)

    if not common_corps.exists():
        return []

    return list(alliances)



from datetime import datetime

from ninja import Schema


class Killboard(Schema):
    kills: int | None = None
    date: datetime | None = None
    character_id: int | None = None
    character_name: str | None = None
    ship_id: int | None = None
    ship_name: str | None = None
    hash: str | None = None
    totalValue: int | None = None
    attackers: list | None = None


class KillboardHall(Schema):
    shame: list | None = None
    fame: list | None = None


class KillboardStats(Schema):
    stats: list | None = None


class CorporationAdmin(Schema):
    corporation: dict | None = None


class AllianceAdmin(Schema):
    alliance: dict | None = None



from django.test import TestCase
from django.utils import timezone
from eveuniverse.models import EveEntity, EveGroup, EveType

from allianceauth.eveonline.evelinks import eveimageserver
from allianceauth.eveonline.models import EveCharacter

from killstats.models.killboard import Attacker, Killmail
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all

MODULE_PATH = "killstats.models.killstatsaudit"


class TestAttackertModel(TestCase):
    @classmethod
    def setUp(self):
        load_allianceauth()
        load_killstats_all()
        self.killmail = Killmail.objects.get(killmail_id=119303113)
        self.attacker = Attacker.objects.filter(killmail=self.killmail).first()

    def test_get_or_unknown_ship_name(self):
        self.assertEqual(self.attacker.get_or_unknown_ship_name(), "Attacker Ship I")
        self.attacker.ship = None
        self.assertEqual(self.attacker.get_or_unknown_ship_name(), "Unknown")

    def test_evaluate_attacker_id(self):
        self.assertEqual(self.attacker.evaluate_attacker(), (1000, "CONCORD"))
        self.attacker.character = None
        self.assertEqual(self.attacker.evaluate_attacker(), (1000125, "CONCORD"))
        self.attacker.corporation = None
        self.attacker.alliance = EveEntity.objects.get(id=30000001)
        self.assertEqual(self.attacker.evaluate_attacker(), (30000001, "Voices of War"))
        self.attacker.alliance = None
        self.assertEqual(self.attacker.evaluate_attacker(), (0, "Unknown"))



# Generated by Django 4.2.11 on 2024-09-26 21:34

from django.db import migrations
from eveuniverse.models import EveEntity

from killstats.managers.killmail_core import KillmailManager
from killstats.models.killboard import Killmail


def migrate_table(apps, schema_editor):
    print("\nMigrating Killmails this can take a while...")
    killmails = Killmail.objects.all()
    kms_count = killmails.count()
    runs = 0
    characters = 0
    corporations = 0
    alliances = 0
    for killmail in killmails:
        try:
            if killmail.victim_id:
                _, created = EveEntity.objects.get_or_create_esi(id=killmail.victim_id)
                if created:
                    characters = characters + 1
            if killmail.victim_corporation_id:
                _, created = EveEntity.objects.get_or_create_esi(
                    id=killmail.victim_corporation_id
                )
                if created:
                    corporations = corporations + 1
            if killmail.victim_alliance_id:
                _, created = EveEntity.objects.get_or_create_esi(
                    id=killmail.victim_alliance_id
                )
                if created:
                    alliances = alliances + 1

            killmail_data = {
                "killmail_id": killmail.killmail_id,
                "zkb": {"hash": killmail.hash},
            }
            killmail_dict = KillmailManager._process_killmail_data(killmail_data)
            km = KillmailManager._create_from_dict(killmail_dict)
            km.create_attackers(killmail, km)
            runs = runs + 1
        except Exception as e:
            print(
                f"Error processing killmail {killmail.killmail_id} deleting killmail..."
            )
            killmail.delete()
            continue
    print(f"{characters} characters created")
    print(f"{corporations} corporations created")
    print(f"{alliances} alliances created")
    print(f"{runs} from {kms_count} Killmails migrated")


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0010_alter_eveindustryactivityduration_eve_type_and_more"),
        ("killstats", "0003_attacker_remove_killmail_attackers_and_more"),
    ]

    operations = [
        migrations.RunPython(migrate_table),
    ]



from django.test import TestCase

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from app_utils.testing import create_user_from_evecharacter

from killstats.models.killstatsaudit import CorporationsAudit
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all


class KillstatsAuditQuerySetTest(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        load_killstats_all()

    def test_visible_to_superuser(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
        )
        self.user.is_superuser = True
        self.user.save()
        # when
        result = CorporationsAudit.objects.visible_to(self.user)
        expected_result = CorporationsAudit.objects.all()
        # then
        self.assertEqual(list(result), list(expected_result))

    def test_visible_to_admin_access(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.admin_access",
            ],
        )
        # when
        expected_result = CorporationsAudit.objects.all()
        result = CorporationsAudit.objects.visible_to(self.user)
        # then
        self.assertEqual(list(result), list(expected_result))

    def test_visible_to(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
        )
        # when
        expected_result = CorporationsAudit.objects.filter(
            corporation=EveCorporationInfo.objects.get(corporation_id=20000001)
        )
        result = CorporationsAudit.objects.visible_to(self.user)
        # then
        self.assertEqual(list(result), list(expected_result))

    def test_visible_to_error(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
        )
        self.user.profile.main_character = None
        # when
        result = list(
            CorporationsAudit.objects.visible_to(self.user).values_list(
                "corporation_id", flat=True
            )
        )
        # then
        self.assertEqual(result, [])



"""API"""

from ninja import NinjaAPI

from django.http import JsonResponse
from django.shortcuts import render

# AA Killstats
from killstats.api.killstats import api_helper
from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class KillboardStatsApiEndpoints:
    tags = ["Stats"]

    # pylint: disable=too-many-locals
    def __init__(self, api: NinjaAPI):
        self.register_endpoints(api)

    # pylint: disable=too-many-statements
    def register_endpoints(self, api: NinjaAPI):
        @api.get(
            "stats/top/victim/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_victim_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_victim_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                top_victim = api_helper.get_top_victim(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": top_victim,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/top/killer/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_killer_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_killer_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                top_killer = api_helper.get_top_killer(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": top_killer,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/top/alltime_victim/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        # pylint: disable=unused-argument
        def get_alltime_victim_api(
            request, month, year, entity_type: str, entity_id: int
        ):
            cache_name = f"alltime_victim_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                alltime_victim = api_helper.get_alltime_victim(
                    request=request,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": alltime_victim,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/top/alltime_killer/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        # pylint: disable=unused-argument
        def get_alltime_killer_api(
            request, month, year, entity_type: str, entity_id: int
        ):
            cache_name = f"alltime_killer_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                alltime_killer = api_helper.get_alltime_killer(
                    request=request,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": alltime_killer,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/ship/worst/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_worst_ship_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"worst_ship_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                worst_ship = api_helper.get_worst_ship(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": worst_ship,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/ship/top/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_ship_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_ship_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                top_ship = api_helper.get_top_ship(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": top_ship,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/top/kill/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_kill_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_kill_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                top_kill = api_helper.get_top_kill(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": top_kill,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/top/loss/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_loss_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_loss_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                top_loss = api_helper.get_top_loss(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = []
                output.append(
                    {
                        "stats": top_loss,
                    }
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)

        @api.get(
            "stats/top/10/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_10_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_10_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                top_10 = api_helper.get_top_10(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = {"top10": top_10}
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return render(
                request,
                "killstats/partials/information/view_top_10.html",
                output,
            )



# Generated by Django 4.2.11 on 2024-11-22 17:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("killstats", "0005_alter_general_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="attacker",
            name="weapon_type_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="killmail",
            name="killmail_date",
            field=models.DateTimeField(blank=True, max_length=0, null=True),
        ),
        migrations.AlterField(
            model_name="killmail",
            name="killmail_id",
            field=models.PositiveIntegerField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="killmail",
            name="victim_alliance_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="killmail",
            name="victim_corporation_id",
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name="killmail",
            name="victim_region_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="killmail",
            name="victim_solar_system_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



from datetime import timedelta
from unittest.mock import patch

from django.contrib.auth.models import Permission
from django.test import TestCase
from django.utils import timezone
from eveuniverse.models import EveEntity

from killstats.models.general import General

MODULE_PATH = "killstats.models.general"


class TestGeneralModel(TestCase):
    @classmethod
    def setUp(self):
        pass

    def test_basic_permission(self):
        # given
        permission = Permission.objects.select_related("content_type").get(
            content_type__app_label="killstats", codename="basic_access"
        )
        # when
        self.assertEqual(General.basic_permission(), permission)

    def test_users_with_basic_access(self):
        with patch(
            MODULE_PATH + ".users_with_permission"
        ) as mock_users_with_permission:
            mock_users_with_permission.return_value = "Test"
            self.assertEqual(General.users_with_basic_access(), "Test")



"""Admin models"""

from django_redis import get_redis_connection

from django.contrib import admin
from django.utils.html import format_html

from allianceauth.eveonline.evelinks import eveimageserver

from killstats.hooks import get_extension_logger
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit

logger = get_extension_logger(__name__)


@admin.register(CorporationsAudit)
class CorporationsAuditAdmin(admin.ModelAdmin):
    list_display = (
        "_entity_pic",
        "_corporation__corporation_id",
        "_last_update",
    )

    list_display_links = (
        "_entity_pic",
        "_corporation__corporation_id",
    )

    list_select_related = ("corporation",)

    ordering = ["corporation__corporation_name"]

    search_fields = ["corporation__corporation_name", "corporation__corporation_id"]

    actions = ["delete_objects", "clear_cache_for_selected"]

    @admin.display(description="")
    def _entity_pic(self, obj: CorporationsAudit):
        eve_id = obj.corporation.corporation_id
        return format_html(
            '<img src="{}" class="img-circle">',
            eveimageserver._eve_entity_image_url("corporation", eve_id, 32),
        )

    @admin.display(description="Corporation ID", ordering="corporation__corporation_id")
    def _corporation__corporation_id(self, obj: CorporationsAudit):
        return obj.corporation.corporation_id

    @admin.display(description="Last Update", ordering="last_update")
    def _last_update(self, obj: CorporationsAudit):
        return obj.last_update

    # pylint: disable=unused-argument
    def has_add_permission(self, request):
        return False

    # pylint: disable=unused-argument
    def has_change_permission(self, request, obj=None):
        return False

    @admin.action(description="Clear cache for selected Corporations")
    def clear_cache_for_selected(self, request, queryset: CorporationsAudit):
        for obj in queryset:
            clear_cache_zkb(obj.corporation.corporation_id)
            logger.debug("Clearing cache for selected Corporations %s", obj.corporation)
        self.message_user(
            request, "Cache successfully cleared for selected Corporations."
        )


@admin.register(AlliancesAudit)
class AlliancesAuditAdmin(admin.ModelAdmin):
    list_display = (
        "_entity_pic",
        "_alliance__alliance_id",
        "_last_update",
    )

    list_display_links = (
        "_entity_pic",
        "_alliance__alliance_id",
    )

    list_select_related = ("alliance",)

    ordering = ["alliance__alliance_name"]

    search_fields = ["alliance__alliance_name", "alliance__alliance_id"]

    actions = ["delete_objects", "clear_cache_for_selected"]

    @admin.display(description="")
    def _entity_pic(self, obj: AlliancesAudit):
        eve_id = obj.alliance.alliance_id
        return format_html(
            '<img src="{}" class="img-circle">',
            eveimageserver._eve_entity_image_url("alliance", eve_id, 32),
        )

    @admin.display(description="Alliance ID", ordering="alliance__alliance_id")
    def _alliance__alliance_id(self, obj: AlliancesAudit):
        return obj.alliance.alliance_id

    @admin.display(description="Last Update", ordering="last_update")
    def _last_update(self, obj: AlliancesAudit):
        return obj.last_update

    # pylint: disable=unused-argument
    def has_add_permission(self, request):
        return False

    # pylint: disable=unused-argument
    def has_change_permission(self, request, obj=None):
        return False

    @admin.action(description="Clear cache for selected Alliances")
    def clear_cache_for_selected(self, request, queryset: AlliancesAudit):
        for obj in queryset:
            clear_alliance_cache_zkb(obj.alliance.alliance_id)
            logger.debug("Clearing cache for selected Alliances %s", obj.alliance)
        self.message_user(request, "Cache successfully cleared for selected Alliances.")


def clear_cache_zkb(corporation_id: int):
    conn = get_redis_connection("default")
    for key in conn.scan_iter(
        f"*:zkb_page_cache_https://zkillboard.com/api/npc/0/corporationID/{corporation_id}*"
    ):
        conn.delete(key)
        logger.debug("Deleting key %s", key)
    return True


def clear_alliance_cache_zkb(alliance_id: int):
    conn = get_redis_connection("default")
    for key in conn.scan_iter(
        f"*:zkb_page_cache_https://zkillboard.com/api/npc/0/allianceID/{alliance_id}*"
    ):
        conn.delete(key)
        logger.debug("Deleting key %s", key)
    return True



Killstats_Stats = [{"stats": []}]

Killstats_Stats_Entry = [
    {
        "stats": [
            {
                "killmail_id": 119303073,
                "character_id": 1002,
                "character_name": "rotze Rotineque",
                "corporation_id": 20000001,
                "alliance_id": 30000001,
                "ship": 10001,
                "ship_name": "Victim Ship I",
                "hash": "VictimHash2",
                "totalValue": 94395699,
                "date": "2024-07-07T21:36:58+00:00",
                "title": "Top Loss:",
                "portrait": "https://images.evetech.net/types/10001/icon?size=256",
                "zkb_link": "https://zkillboard.com/character/1002/",
            },
            {
                "title": "Worst Ship:",
                "type": "count",
                "ship_id": 10001,
                "ship_name": "Victim Ship I",
                "portrait": "https://images.evetech.net/types/10001/icon?size=256",
                "count": 2,
                "loss": True,
                "zkb_link": "https://zkillboard.com/ship/10001/",
            },
            {
                "title": "Top Victim: Garthd",
                "type": "count",
                "character_id": 1004,
                "character_name": "Garthd",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "count": 1,
                "loss": True,
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "title": "Alltime Victim: Garthd",
                "type": "count",
                "character_id": 1004,
                "character_name": "Garthd",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "count": 1,
                "loss": True,
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "killmail_id": 119303075,
                "character_id": 1004,
                "character_name": "Garthd",
                "corporation_id": 20000001,
                "alliance_id": 30000001,
                "ship": 10002,
                "ship_name": "Victim Ship II",
                "hash": "VictimHash3",
                "totalValue": 9999,
                "date": "2024-07-07T21:38:58+00:00",
                "title": "Top Kill:",
                "portrait": "https://images.evetech.net/types/10002/icon?size=256",
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "title": "Top Ship:",
                "type": "count",
                "ship_id": 20002,
                "ship_name": "Attacker Ship II",
                "portrait": "https://images.evetech.net/types/20002/icon?size=256",
                "count": 10,
                "loss": False,
                "zkb_link": "https://zkillboard.com/ship/20002/",
            },
            {
                "title": "Top Killer: Gneuten",
                "type": "count",
                "character_id": 1001,
                "character_name": "Gneuten",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "count": 3,
                "loss": False,
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
            {
                "title": "Alltime Killer: Gneuten",
                "type": "count",
                "character_id": 1001,
                "character_name": "Gneuten",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "count": 3,
                "loss": False,
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
        ]
    }
]

Killstats_Stats_Single = [
    {
        "stats": [
            {
                "killmail_id": 119303073,
                "character_id": 1002,
                "character_name": "rotze Rotineque",
                "corporation_id": 20000001,
                "alliance_id": 30000001,
                "ship": 10001,
                "ship_name": "Victim Ship I",
                "hash": "VictimHash2",
                "totalValue": 94395699,
                "date": "2024-07-07T21:36:58+00:00",
                "title": "Top Loss:",
                "portrait": "https://images.evetech.net/types/10001/icon?size=256",
                "zkb_link": "https://zkillboard.com/character/1002/",
            },
            {
                "title": "Worst Ship:",
                "type": "count",
                "ship_id": 10001,
                "ship_name": "Victim Ship I",
                "portrait": "https://images.evetech.net/types/10001/icon?size=256",
                "count": 2,
                "loss": True,
                "zkb_link": "https://zkillboard.com/ship/10001/",
            },
            {
                "title": "Top Victim: Garthd",
                "type": "count",
                "character_id": 1004,
                "character_name": "Garthd",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "count": 1,
                "loss": True,
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "title": "Alltime Victim: Garthd",
                "type": "count",
                "character_id": 1004,
                "character_name": "Garthd",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "count": 1,
                "loss": True,
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "killmail_id": 119303075,
                "character_id": 1004,
                "character_name": "Garthd",
                "corporation_id": 20000001,
                "alliance_id": 30000001,
                "ship": 10002,
                "ship_name": "Victim Ship II",
                "hash": "VictimHash3",
                "totalValue": 9999,
                "date": "2024-07-07T21:38:58+00:00",
                "title": "Top Kill:",
                "portrait": "https://images.evetech.net/types/10002/icon?size=256",
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "title": "Top Ship:",
                "type": "count",
                "ship_id": 20002,
                "ship_name": "Attacker Ship II",
                "portrait": "https://images.evetech.net/types/20002/icon?size=256",
                "count": 10,
                "loss": False,
                "zkb_link": "https://zkillboard.com/ship/20002/",
            },
            {
                "title": "Top Killer: Gneuten",
                "type": "count",
                "character_id": 1001,
                "character_name": "Gneuten",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "count": 3,
                "loss": False,
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
            {
                "title": "Alltime Killer: Gneuten",
                "type": "count",
                "character_id": 1001,
                "character_name": "Gneuten",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "count": 3,
                "loss": False,
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
        ]
    }
]


Killstats_Halls_Entry = [
    {
        "shame": [
            {
                "killmail_id": 119303073,
                "character_id": 1002,
                "character_name": "rotze Rotineque",
                "ship": 10001,
                "ship_name": "Victim Ship I",
                "totalValue": 94395699,
                "title": "rotze Rotineque",
                "portrait": "https://images.evetech.net/characters/1002/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1002/",
            },
            {
                "killmail_id": 119303113,
                "character_id": 1001,
                "character_name": "Gneuten",
                "ship": 10001,
                "ship_name": "Victim Ship I",
                "totalValue": 76734851,
                "title": "Gneuten",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
            {
                "killmail_id": 119303075,
                "character_id": 1004,
                "character_name": "Garthd",
                "ship": 10002,
                "ship_name": "Victim Ship II",
                "totalValue": 9999,
                "title": "Garthd",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
        ],
        "fame": [
            {
                "killmail_id": 119303075,
                "character_id": 1001,
                "character_name": "Gneuten",
                "ship": 10002,
                "ship_name": "Victim Ship II",
                "totalValue": 9999,
                "title": "Garthd",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
            {
                "killmail_id": 1,
                "character_id": 1002,
                "character_name": "rotze Rotineque",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1002/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1002/",
            },
            {
                "killmail_id": 2,
                "character_id": 1004,
                "character_name": "Garthd",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "killmail_id": 3,
                "character_id": 1012,
                "character_name": "Test3",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1012/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1012/",
            },
            {
                "killmail_id": 4,
                "character_id": 1010,
                "character_name": "Test1",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1010/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1010/",
            },
        ],
    }
]

Killstats_Halls_Single = [
    {
        "shame": [
            {
                "killmail_id": 119303073,
                "character_id": 1002,
                "character_name": "rotze Rotineque",
                "ship": 10001,
                "ship_name": "Victim Ship I",
                "totalValue": 94395699,
                "title": "rotze Rotineque",
                "portrait": "https://images.evetech.net/characters/1002/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1002/",
            },
            {
                "killmail_id": 119303113,
                "character_id": 1001,
                "character_name": "Gneuten",
                "ship": 10001,
                "ship_name": "Victim Ship I",
                "totalValue": 76734851,
                "title": "Gneuten",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
            {
                "killmail_id": 119303075,
                "character_id": 1004,
                "character_name": "Garthd",
                "ship": 10002,
                "ship_name": "Victim Ship II",
                "totalValue": 9999,
                "title": "Garthd",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
        ],
        "fame": [
            {
                "killmail_id": 119303075,
                "character_id": 1001,
                "character_name": "Gneuten",
                "ship": 10002,
                "ship_name": "Victim Ship II",
                "totalValue": 9999,
                "title": "Garthd",
                "portrait": "https://images.evetech.net/characters/1001/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1001/",
            },
            {
                "killmail_id": 1,
                "character_id": 1002,
                "character_name": "rotze Rotineque",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1002/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1002/",
            },
            {
                "killmail_id": 2,
                "character_id": 1004,
                "character_name": "Garthd",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1004/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1004/",
            },
            {
                "killmail_id": 3,
                "character_id": 1012,
                "character_name": "Test3",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1012/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1012/",
            },
            {
                "killmail_id": 4,
                "character_id": 1010,
                "character_name": "Test1",
                "ship": 10003,
                "ship_name": "Victim Ship III",
                "totalValue": 1000,
                "title": "CONCORD",
                "portrait": "https://images.evetech.net/characters/1010/portrait?size=256",
                "zkb_link": "https://zkillboard.com/character/1010/",
            },
        ],
    }
]

Killstats_Kills_Entry = {
    "draw": 1,
    "recordsTotal": 6,
    "recordsFiltered": 6,
    "data": [
        {
            "killmail_id": 119303077,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1000, "name": "CONCORD"},
            "victim_ship": {"id": 10003, "name": "Victim Ship III"},
            "victim_corporation_id": 1000125,
            "victim_alliance_id": None,
            "hash": "AttackerHash1",
            "victim_total_value": 1000,
            "victim_fitted_value": 2000,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
        {
            "killmail_id": 119303075,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1004, "name": "Garthd"},
            "victim_ship": {"id": 10002, "name": "Victim Ship II"},
            "victim_corporation_id": 20000001,
            "victim_alliance_id": 30000001,
            "hash": "VictimHash3",
            "victim_total_value": 9999,
            "victim_fitted_value": 9888,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
        {
            "killmail_id": 4,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1000, "name": "CONCORD"},
            "victim_ship": {"id": 10003, "name": "Victim Ship III"},
            "victim_corporation_id": 1000125,
            "victim_alliance_id": None,
            "hash": "AttackerHash5",
            "victim_total_value": 1000,
            "victim_fitted_value": 2000,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
        {
            "killmail_id": 3,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1000, "name": "CONCORD"},
            "victim_ship": {"id": 10003, "name": "Victim Ship III"},
            "victim_corporation_id": 1000125,
            "victim_alliance_id": None,
            "hash": "AttackerHash4",
            "victim_total_value": 1000,
            "victim_fitted_value": 2000,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
        {
            "killmail_id": 2,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1000, "name": "CONCORD"},
            "victim_ship": {"id": 10003, "name": "Victim Ship III"},
            "victim_corporation_id": 1000125,
            "victim_alliance_id": None,
            "hash": "AttackerHash3",
            "victim_total_value": 1000,
            "victim_fitted_value": 2000,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
        {
            "killmail_id": 1,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1000, "name": "CONCORD"},
            "victim_ship": {"id": 10003, "name": "Victim Ship III"},
            "victim_corporation_id": 1000125,
            "victim_alliance_id": None,
            "hash": "AttackerHash2",
            "victim_total_value": 1000,
            "victim_fitted_value": 2000,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
    ],
    "totalvalue": [14999],
}

Killstats_Losses_Entry = {
    "draw": 1,
    "recordsTotal": 3,
    "recordsFiltered": 3,
    "data": [
        {
            "killmail_id": 119303113,
            "killmail_date": "2024-07-07T21:38:59Z",
            "victim": {"id": 1001, "name": "Gneuten"},
            "victim_ship": {"id": 10001, "name": "Victim Ship I"},
            "victim_corporation_id": 20000001,
            "victim_alliance_id": 30000001,
            "hash": "VictimHash1",
            "victim_total_value": 76734851,
            "victim_fitted_value": 72603992,
            "victim_destroyed_value": 64460362,
            "victim_dropped_value": 12274488,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459026975.88434,
            "victim_position_y": -4921955322.159037,
            "victim_position_z": -8888754631.141735,
        },
        {
            "killmail_id": 119303075,
            "killmail_date": "2024-07-07T21:38:58Z",
            "victim": {"id": 1004, "name": "Garthd"},
            "victim_ship": {"id": 10002, "name": "Victim Ship II"},
            "victim_corporation_id": 20000001,
            "victim_alliance_id": 30000001,
            "hash": "VictimHash3",
            "victim_total_value": 9999,
            "victim_fitted_value": 9888,
            "victim_destroyed_value": 3000,
            "victim_dropped_value": 1000,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
        {
            "killmail_id": 119303073,
            "killmail_date": "2024-07-07T21:36:58Z",
            "victim": {"id": 1002, "name": "rotze Rotineque"},
            "victim_ship": {"id": 10001, "name": "Victim Ship I"},
            "victim_corporation_id": 20000001,
            "victim_alliance_id": 30000001,
            "hash": "VictimHash2",
            "victim_total_value": 94395699,
            "victim_fitted_value": 80172608,
            "victim_destroyed_value": 87869532,
            "victim_dropped_value": 6526166,
            "victim_region_id": 10000059,
            "victim_solar_system_id": 30004668,
            "victim_position_x": -74459258618.29333,
            "victim_position_y": -4921962525.529236,
            "victim_position_z": -8888733393.073042,
        },
    ],
    "totalvalue": [171140549],
}



from unittest.mock import MagicMock, patch

from django.test import TestCase

from app_utils.testing import create_user_from_evecharacter

from killstats.api.killboard_manager import _get_character_details_victim
from killstats.api.killstats.api_helper import get_killstats_halls
from killstats.models.killboard import Killmail
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all

MODULE_PATH = "killstats.api.killstats.api_helper"
MODULE_PATH2 = "killstats.api.killboard_manager"


class Test_KillboardEndpoints(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_killstats_all()

        cls.user, _ = create_user_from_evecharacter(
            1001,
            permissions=["killstats.basic_access", "killstats.admin_access"],
        )

        cls.user2, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "killstats.basic_access",
            ],
        )

        cls.killmail = Killmail.objects.get(killmail_id=119303113)

    def test_get_killstats_halls_filter_corporations(self):
        request = MagicMock()
        request.user = self.user
        month = 1
        year = 2023
        entity_id = 0
        entity_type = "corporation"

        with patch(MODULE_PATH + ".get_entities", return_value=[20000001, 10000001]):
            halls = get_killstats_halls(request, month, year, entity_type, entity_id)
            self.assertIsNotNone(halls)
            self.assertIn("shame", halls[0])
            self.assertIn("fame", halls[0])
            self.assertNotIn(10000001, halls[0]["shame"])
            self.assertNotIn(10000001, halls[0]["fame"])

    def test_get_killstats_halls_filter_alliances(self):
        request = MagicMock()
        request.user = self.user
        month = 7
        year = 2024
        entity_id = 0
        entity_type = "alliance"

        halls = get_killstats_halls(request, month, year, entity_type, entity_id)

        self.assertIsNotNone(halls)
        self.assertIn("shame", halls[0])
        self.assertIn("fame", halls[0])
        self.assertNotIn(10000001, halls[0]["shame"])
        self.assertNotIn(10000001, halls[0]["fame"])

    def test_get_killstats_halls_filter_corporations_single(self):
        request = MagicMock()
        request.user = self.user
        month = 7
        year = 2024
        entity_id = 20000001
        entity_type = "corporation"

        halls = get_killstats_halls(request, month, year, entity_type, entity_id)
        self.assertIsNotNone(halls)
        self.assertIn("shame", halls[0])
        self.assertIn("fame", halls[0])
        self.assertNotIn(10000001, halls[0]["shame"])
        self.assertNotIn(10000001, halls[0]["fame"])

    def test_get_killstats_halls_filter_no_enitity(self):
        request = MagicMock()
        request.user = self.user
        month = 7
        year = 2024
        entity_id = 13770000
        entity_type = "corporation"

        halls = get_killstats_halls(request, month, year, entity_type, entity_id)

        self.assertIsNotNone(halls)
        self.assertIn("shame", halls[0])
        self.assertIn("fame", halls[0])
        self.assertNotIn(10000001, halls[0]["shame"])
        self.assertNotIn(10000001, halls[0]["fame"])

    def test_get_character_details_victim_attribute_error(self):
        mains = MagicMock()

        # Konfiguriere den MagicMock so, dass er einen AttributeError auslöst, wenn auf mains.values() zugegriffen wird
        mains.values.side_effect = AttributeError

        character_id, character_name = _get_character_details_victim(
            self.killmail, mains
        )

        self.assertEqual(character_id, 1001)
        self.assertEqual(character_name, "Gneuten")



# Standard Library
import json
import time
from dataclasses import asdict, dataclass
from datetime import datetime
from typing import Optional

# Third Party
import requests
from dacite import DaciteError, from_dict
from redis.exceptions import LockError

# Django
from django.core.cache import cache
from django.utils.dateparse import parse_datetime

# Alliance Auth (External Libs)
from app_utils.allianceauth import get_redis_client
from app_utils.json import JSONDateTimeDecoder, JSONDateTimeEncoder

# AA Voices of War
from killstats import __title__, __version__
from killstats.app_settings import (
    KILLBOARD_MAX_KILLMAILS_PER_RUN,
    KILLBOARD_STORAGE_LIFETIME,
    KILLBOARD_ZKB_CACHE_LIFETIME,
    STORAGE_BASE_KEY,
    ZKILLBOARD_API_URL,
)
from killstats.hooks import get_extension_logger
from killstats.providers import esi

logger = get_extension_logger(__name__)
USERAGENT = "killstats v{__version__}"
RETRY_DELAY = 3


class KillboardException(Exception):
    """Exception from Killboard"""


class KillmailDoesNotExist(KillboardException):
    """Killmail does not exist in storage."""


@dataclass
class _KillmailBase:
    """Base class for all Killmail."""

    def asdict(self) -> dict:
        """Return this object as dict."""
        return asdict(self)


@dataclass
class _KillmailCharacter(_KillmailBase):
    ENTITY_PROPS = [
        "character_id",
        "corporation_id",
        "alliance_id",
        "faction_id",
        "ship_type_id",
    ]

    character_id: int | None = None
    corporation_id: int | None = None
    alliance_id: int | None = None
    faction_id: int | None = None
    ship_type_id: int | None = None


@dataclass
class KillmailVictim(_KillmailCharacter):
    """A victim on a killmail."""

    damage_taken: int | None = None


@dataclass
class KillmailAttacker(_KillmailCharacter):
    """An attacker on a killmail."""

    ENTITY_PROPS = _KillmailCharacter.ENTITY_PROPS + ["weapon_type_id"]

    damage_done: int | None = None
    is_final_blow: bool | None = None
    security_status: float | None = None
    weapon_type_id: int | None = None


@dataclass
class KillmailPosition(_KillmailBase):
    "A position for a killmail."
    x: float | None = None
    y: float | None = None
    z: float | None = None


@dataclass
class KillmailZkb(_KillmailBase):
    """A ZKB entry for a killmail."""

    location_id: int | None = None
    hash: str | None = None
    fitted_value: float | None = None
    dropped_value: float | None = None
    destroyed_value: float | None = None
    total_value: float | None = None
    points: int | None = None
    is_npc: bool | None = None
    is_solo: bool | None = None
    is_awox: bool | None = None


@dataclass
class KillmailManager(_KillmailBase):
    """Killmail Manager"""

    _STORAGE_BASE_KEY = "killboard_storage_killmail_"

    id: int
    time: datetime
    victim: KillmailVictim
    attackers: list[KillmailAttacker]
    position: KillmailPosition
    zkb: KillmailZkb
    solar_system_id: int | None = None

    def __repr__(self):
        return f"{type(self).__name__}(id={self.id})"

    def asjson(self) -> str:
        """Convert killmail into JSON data."""
        return json.dumps(asdict(self), cls=JSONDateTimeEncoder)

    @staticmethod
    def get_killmail_data_bulk(
        base_url: str, max_entries_per_bulk: int = KILLBOARD_MAX_KILLMAILS_PER_RUN
    ):
        """Get kill data bulk from zKillboard"""
        # TODO Maybe another way to get the data?
        # pylint: disable=import-outside-toplevel
        from killstats.models.killboard import Killmail

        killmail_list = []
        try:
            for page in range(1, 6):
                result = KillmailManager._fetch_page_data(base_url, page)
                if result is None:
                    continue
                killmail_ids = [data["killmail_id"] for data in result]
                existing_killmail_ids = Killmail.objects.filter(
                    killmail_id__in=killmail_ids
                ).values_list("killmail_id", flat=True)
                new_killmails = [
                    data
                    for data in result
                    if data["killmail_id"] not in existing_killmail_ids
                ]

                for data in new_killmails:
                    killmail_dict = KillmailManager._process_killmail_data(data)
                    if killmail_dict:
                        killmail_list.append(killmail_dict)
                        if len(killmail_list) >= max_entries_per_bulk:
                            return killmail_list
        # pylint: disable=broad-exception-caught
        except Exception as exc:
            logger.error("Failed to fetch killmails: %s", exc, exc_info=True)
        return killmail_list

    @staticmethod
    def _fetch_page_data(base_url, page):
        cache_key = f"zkb_page_cache_{base_url}_{page}"
        cached_zkb = cache.get(cache_key)
        # Check only the first page if they changed and cache lifetime is not expired
        if cached_zkb is not None and page > 3:
            return cached_zkb

        redis = get_redis_client()

        try:
            # Ensure that you dont spam zKB Requests
            with redis.lock(KillmailManager.lock_key(), blocking_timeout=600):
                logger.debug("Fetching %s page %s from zKillboard", base_url, page)
                url = f"{base_url}page/{page}/"
                headers = {
                    "User-Agent": USERAGENT,
                    "Content-Type": "application/json",
                    "Accept-Encoding": "gzip",
                }

                timeout_value = KILLBOARD_ZKB_CACHE_LIFETIME

                # Make a GET request to fetch the new content
                request_result = requests.get(url=url, headers=headers)
                request_result.raise_for_status()
                new_data = request_result.json()

                cache.set(
                    key=cache_key,
                    value=new_data,
                    timeout=timeout_value,
                )
                # Sleep to prevent spamming zKB
                time.sleep(1)
                return request_result.json()
        except LockError:
            logger.debug("Lock is already in use.")
        except requests.RequestException as exc:
            logger.warning("Request failed: %s", exc, exc_info=True)
            raise ValueError(str(exc)) from exc
        logger.error("Failed to fetch %s page %s from zKillboard", base_url, page)
        return None

    @staticmethod
    def _process_killmail_data(data):
        try:
            killmail_id = data["killmail_id"]
            killmail_hash = data["zkb"]["hash"]
            esi_killmail = esi.client.Killmails.get_killmails_killmail_id_killmail_hash(
                killmail_id=killmail_id, killmail_hash=killmail_hash
            ).result()
            esi_killmail["killmail_time"] = esi_killmail["killmail_time"].strftime(
                "%Y-%m-%dT%H:%M:%SZ"
            )
            return {"killID": killmail_id, "killmail": esi_killmail, "zkb": data["zkb"]}
        # pylint: disable=broad-exception-caught
        except Exception as exc:
            logger.error("Killmail can't fetch %s", exc)
            return None

    # pylint: disable=too-many-locals
    @classmethod
    def get_single_killmail(cls, killmail_id: int):
        """Get kill data from zKillboard"""
        # pylint: disable=import-outside-toplevel
        from killstats.models.killboard import Killmail

        cache_key = f"{STORAGE_BASE_KEY}_KILLMAIL_{killmail_id}"
        killmail_json = cache.get(cache_key)
        if killmail_json:
            return KillmailManager.from_json(killmail_json)

        logger.debug("Fetching killmail %s from zKillboard", killmail_id)

        url = f"{ZKILLBOARD_API_URL}killID/{killmail_id}/"
        headers = {"User-Agent": USERAGENT, "Content-Type": "application/json"}
        request_result = requests.get(url=url, headers=headers, timeout=5)

        try:
            request_result.raise_for_status()
            zkb_killmail = request_result.json()[0]
        except requests.HTTPError as exc:
            error_str = str(exc)

            logger.warning(
                msg=f"Unable to get killmail details from zKillboard. Error: {error_str}",
                exc_info=True,
            )

            raise ValueError(error_str) from exc
        except requests.Timeout as exc:
            error_str = str(exc)

            logger.warning(msg="Connection to zKillboard timed out …")

            raise ValueError(error_str) from exc

        try:
            killmail_id = zkb_killmail["killmail_id"]
            killmail_hash = zkb_killmail["zkb"]["hash"]

            # Überprüfe, ob die Killmail bereits in der Datenbank existiert
            existing_killmail = Killmail.objects.filter(killmail_id=killmail_id).first()

            if existing_killmail:
                logger.debug("Killmail %s exists already..", killmail_id)
                return None

            esi_killmail = esi.client.Killmails.get_killmails_killmail_id_killmail_hash(
                killmail_id=killmail_id, killmail_hash=killmail_hash
            ).results()
        except Exception as exc:
            raise ValueError("Invalid Kill ID or Hash.") from exc

        esi_killmail["killmail_time"] = esi_killmail["killmail_time"].strftime(
            "%Y-%m-%dT%H:%M:%SZ"
        )

        killmail_dict = {
            "killID": killmail_id,
            "killmail": esi_killmail,
            "zkb": zkb_killmail["zkb"],
        }
        killmail = cls._create_from_dict(killmail_dict)
        if killmail:
            cache.set(key=cache_key, value=killmail.asjson())
        return killmail

    def save(self) -> None:
        """Save this killmail to temporary storage."""
        cache.set(
            key=self._storage_key(self.id),
            value=self.asjson(),
            timeout=KILLBOARD_STORAGE_LIFETIME,
        )
        logger.debug("Cache created for %s", self.id)

    def delete(self) -> None:
        """Delete this killmail from temporary storage."""
        cache.delete(self._storage_key(self.id))

    def create_names_bulk(self, eve_ids: list):
        # pylint: disable=import-outside-toplevel
        from eveuniverse.models import EveEntity

        if len(eve_ids) > 0:
            EveEntity.objects.bulk_create_esi(eve_ids)
            return True
        return False

    @staticmethod
    def get_ship_name(ship_type: int):
        # pylint: disable=import-outside-toplevel
        from eveuniverse.models import EveType

        ship_name, new_entry = EveType.objects.get_or_create_esi(id=ship_type)
        if new_entry:
            logger.debug("Kllboard Manager Ship: %s added", ship_name.name)
        return ship_name

    @staticmethod
    def get_entity_name(eve_id: int):
        # pylint: disable=import-outside-toplevel
        from eveuniverse.models import EveEntity

        entity, new_entry = EveEntity.objects.get_or_create_esi(id=eve_id)
        if new_entry:
            logger.debug("Kllboard Manager EveName: %s added", entity.name)
        return entity

    @staticmethod
    def get_region_id(solar_system: int):
        # pylint: disable=import-outside-toplevel
        from eveuniverse.models import EveSolarSystem

        solar_system, new_entry = EveSolarSystem.objects.get_or_create_esi(
            id=solar_system
        )
        region_id = solar_system.eve_constellation.eve_region
        if new_entry:
            logger.debug("%s added to System", region_id.name)
        return region_id.id

    def create_attackers(self, killmail, killmanager):
        # pylint: disable=import-outside-toplevel
        from killstats.models.killboard import Attacker

        for attacker in killmanager.attackers:
            character = None
            if attacker.character_id:
                character = self.get_entity_name(attacker.character_id)

            corporation = None
            if attacker.corporation_id:
                corporation = self.get_entity_name(attacker.corporation_id)

            alliance = None
            if attacker.alliance_id:
                alliance = self.get_entity_name(attacker.alliance_id)

            ship = None
            if attacker.ship_type_id:
                ship = self.get_ship_name(attacker.ship_type_id)

            Attacker.objects.get_or_create(
                killmail=killmail,
                character=character,
                corporation=corporation,
                alliance=alliance,
                defaults={
                    "ship": ship,
                    "damage_done": attacker.damage_done,
                    "final_blow": attacker.is_final_blow,
                    "security_status": attacker.security_status,
                    "weapon_type_id": attacker.weapon_type_id,
                },
            )
        return True

    @classmethod
    def get(cls, cache_id: int) -> "KillmailManager":
        """Fetch a killmail from temporary storage."""
        data = cache.get(key=cls._storage_key(cache_id))
        if not data:
            raise KillmailDoesNotExist(
                f"Killmail with ID {cache_id} does not exist in storage."
            )
        return cls.from_json(data)

    @classmethod
    def _storage_key(cls, cache_id: int) -> str:
        return cls._STORAGE_BASE_KEY + str(cache_id)

    @staticmethod
    def lock_key() -> str:
        """Key used for lock operation on Redis."""
        return f"{__title__.upper()}_REDISQ_LOCK"

    @classmethod
    def from_dict(cls, data: dict) -> "KillmailManager":
        """Create new object from dictionary."""
        try:
            return from_dict(data_class=KillmailManager, data=data)
        except DaciteError as ex:
            logger.error("Failed to convert dict to %s", type(cls), exc_info=True)
            raise ex

    @classmethod
    def from_json(cls, json_str: str) -> "KillmailManager":
        """Create new object from JSON data."""
        return cls.from_dict(json.loads(json_str, cls=JSONDateTimeDecoder))

    @classmethod
    def _extract_victim_and_position(cls, killmail_data: dict):
        victim = KillmailVictim()
        position = KillmailPosition()
        if "victim" in killmail_data:
            victim_data = killmail_data["victim"]
            params = {}
            for prop in KillmailVictim.ENTITY_PROPS + ["damage_taken"]:
                if prop in victim_data:
                    params[prop] = victim_data[prop]

            victim = KillmailVictim(**params)

            if "position" in victim_data:
                position_data = victim_data["position"]
                params = {}
                for prop in ["x", "y", "z"]:
                    if prop in position_data:
                        params[prop] = position_data[prop]

                position = KillmailPosition(**params)

        return victim, position

    @classmethod
    def _extract_attackers(cls, killmail_data: dict) -> list[KillmailAttacker]:
        attackers = []
        for attacker_data in killmail_data.get("attackers", []):
            params = {}
            for prop in KillmailAttacker.ENTITY_PROPS + [
                "damage_done",
                "security_status",
            ]:
                if prop in attacker_data:
                    params[prop] = attacker_data[prop]

            if "final_blow" in attacker_data:
                params["is_final_blow"] = attacker_data["final_blow"]

            attackers.append(KillmailAttacker(**params))
        return attackers

    @classmethod
    def _extract_zkb(cls, package_data):
        if "zkb" not in package_data:
            return KillmailZkb()

        zkb_data = package_data["zkb"]
        params = {}
        for prop, mapping in (
            ("locationID", "location_id"),
            ("hash", None),
            ("fittedValue", "fitted_value"),
            ("droppedValue", "dropped_value"),
            ("destroyedValue", "destroyed_value"),
            ("totalValue", "total_value"),
            ("points", None),
            ("npc", "is_npc"),
            ("solo", "is_solo"),
            ("awox", "is_awox"),
        ):
            if prop in zkb_data:
                if mapping:
                    params[mapping] = zkb_data[prop]
                else:
                    params[prop] = zkb_data[prop]

        return KillmailZkb(**params)

    @classmethod
    def _create_from_dict(cls, package_data: dict) -> Optional["KillmailManager"]:
        """creates a new object from given dict.
        Needs to confirm with data structure returned from ZKB RedisQ
        """

        killmail = None
        if "killmail" in package_data:
            killmail_data = package_data["killmail"]
            victim, position = cls._extract_victim_and_position(killmail_data)
            attackers = cls._extract_attackers(killmail_data)
            zkb = cls._extract_zkb(package_data)

            params = {
                "id": killmail_data["killmail_id"],
                "time": parse_datetime(killmail_data["killmail_time"]),
                "victim": victim,
                "position": position,
                "attackers": attackers,
                "zkb": zkb,
            }
            if "solar_system_id" in killmail_data:
                params["solar_system_id"] = killmail_data["solar_system_id"]

            killmail = KillmailManager(**params)

        return killmail



# Django
from django.contrib.auth.models import Permission
from django.db import models
from django.utils.translation import gettext_lazy as _

# Alliance Auth (External Libs)
from app_utils.django import users_with_permission

# AA Killstats
from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class General(models.Model):
    """A model defining commonly used properties and methods for Voices of War."""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app, Killstats."),
            ("admin_access", "Has access to all killstats."),
        )

    @classmethod
    def basic_permission(cls):
        """Return basic permission needed to use this app."""
        return Permission.objects.select_related("content_type").get(
            content_type__app_label=cls._meta.app_label, codename="basic_access"
        )

    @classmethod
    def users_with_basic_access(cls) -> models.QuerySet:
        """Return users which have at least basic access to Voices of War."""
        return users_with_permission(cls.basic_permission())



from unittest.mock import MagicMock

from django.test import TestCase
from django.urls import reverse

from app_utils.testdata_factories import UserMainFactory

from killstats.auth_hooks import KillstatsMenuItem


class TestAuthHooks(TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user = UserMainFactory(permissions=["killstats.basic_access"])
        cls.html_menu = f"""
            <li class="d-flex flex-wrap m-2 p-2 pt-0 pb-0 mt-0 mb-0 me-0 pe-0">
                <i class="nav-link fas fa-star fa-fw fa-fw align-self-center me-3 active"></i>
                <a class="nav-link flex-fill align-self-center me-auto active" href="{reverse('killstats:index')}">
                    Killstats
                </a>
            </li>
        """

    def test_menu_hook(self):
        self.client.force_login(self.user)

        response = self.client.get(reverse("killstats:index"))

        self.assertContains(response, self.html_menu, html=True)

    def test_render_returns_empty_string_for_user_without_permission(self):
        # given
        killstats_menu_item = KillstatsMenuItem()
        mock_request = MagicMock()
        mock_request.user.has_perm.return_value = False

        # when
        result = killstats_menu_item.render(mock_request)
        # then
        self.assertEqual(
            result,
            "",
            "Expected render method to return an empty string for users without permission",
        )



"""API"""

from ninja import NinjaAPI

# AA Killstats
from killstats.api import schema
from killstats.hooks import get_extension_logger
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit

logger = get_extension_logger(__name__)


class KillboardAdminApiEndpoints:
    tags = ["KillboardAdmin"]

    # pylint: disable=too-many-locals
    def __init__(self, api: NinjaAPI):
        self.register_endpoints(api)

    def register_endpoints(self, api: NinjaAPI):
        @api.get(
            "killboard/corporation/admin/",
            response={200: list[schema.CorporationAdmin], 403: str},
            tags=self.tags,
        )
        def get_corporation_admin(request):
            corporations = CorporationsAudit.objects.visible_to(request.user)

            if corporations is None:
                return 403, "Permission Denied"

            corporation_dict = {}

            for corporation in corporations:
                # pylint: disable=broad-exception-caught
                try:
                    corporation_dict[corporation.corporation.corporation_id] = {
                        "corporation_id": corporation.corporation.corporation_id,
                        "corporation_name": corporation.corporation.corporation_name,
                    }
                except Exception:
                    continue

            output = []
            output.append({"corporation": corporation_dict})

            return output

        @api.get(
            "killboard/alliance/admin/",
            response={200: list[schema.AllianceAdmin], 403: str},
            tags=self.tags,
        )
        def get_alliance_admin(request):
            alliances = AlliancesAudit.objects.visible_to(request.user)

            if alliances is None:
                return 403, "Permission Denied"

            alliance_dict = {}

            for alliance in alliances:
                # pylint: disable=broad-exception-caught
                try:
                    alliance_dict[alliance.alliance.alliance_id] = {
                        "alliance_id": alliance.alliance.alliance_id,
                        "alliance_name": alliance.alliance.alliance_name,
                    }
                except Exception:
                    continue

            output = []
            output.append({"alliance": alliance_dict})

            return output



from django.test import TestCase
from django.utils import timezone
from eveuniverse.models import EveEntity, EveGroup, EveType

from allianceauth.eveonline.evelinks import eveimageserver
from allianceauth.eveonline.models import EveCharacter

from killstats.models.killboard import Killmail
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_killstats import load_killstats_all

MODULE_PATH = "killstats.models.killstatsaudit"


class TestKillboardtModel(TestCase):
    @classmethod
    def setUp(self):
        load_allianceauth()
        load_killstats_all()
        self.killmail = Killmail.objects.get(killmail_id=119303113)
        self.killmail2 = Killmail.objects.create(
            killmail_id=119303114,
            killmail_date=timezone.now(),
            victim=EveEntity.objects.get(id=1001),
            victim_ship=EveType.objects.get(id=670),
            victim_corporation_id=20000001,
            victim_alliance_id=30000001,
            hash="hash",
            victim_total_value=1000,
            victim_fitted_value=500,
            victim_destroyed_value=500,
            victim_dropped_value=500,
            victim_region_id=1001,
            victim_solar_system_id=20000001,
            victim_position_x=1.0,
            victim_position_y=1.0,
            victim_position_z=1.0,
        )

    def test_is_corp(self):
        self.assertTrue(self.killmail.is_corp([20000001]))
        self.assertFalse(self.killmail2.is_corp([30000001]))

    def test_is_ally(self):
        self.assertFalse(self.killmail.is_alliance([20000001]))
        self.assertTrue(self.killmail2.is_alliance([30000001]))

    def test_is_structure(self):
        self.killmail.victim_ship.eve_group.eve_category_id = 65
        self.assertTrue(self.killmail.is_structure())
        self.killmail.victim_ship.eve_group.eve_category_id = 0
        self.assertFalse(self.killmail.is_structure())

    def test_is_mobile(self):
        self.killmail.victim_ship.eve_group.eve_category_id = 22
        self.assertTrue(self.killmail.is_mobile())
        self.killmail.victim_ship.eve_group.eve_category_id = 0
        self.assertFalse(self.killmail.is_mobile())

    def test_is_capsule(self):
        self.assertFalse(self.killmail.is_capsule())
        self.killmail.victim_ship.eve_group = EveGroup.objects.get(id=29)
        self.assertTrue(self.killmail.is_capsule())

    def test_get_month(self):
        self.killmail.killmail_date = timezone.datetime(
            2023, 10, 1, tzinfo=timezone.utc
        )
        self.assertTrue(self.killmail.get_month(10))
        self.killmail.killmail_date = timezone.datetime(
            2023, 1, 30, tzinfo=timezone.utc
        )
        self.assertFalse(self.killmail.get_month(9))

    def test_threshold(self):
        self.assertTrue(self.killmail.threshold(500_000))
        self.assertFalse(self.killmail.threshold(150_000_000))

    def test_str(self):
        self.assertEqual(str(self.killmail), "Killmail 119303113")

    def test_get_image_url(self):
        expected_url = "https://images.evetech.net/characters/1001/portrait?size=32"
        self.assertEqual(self.killmail.get_image_url(), expected_url)
        self.killmail.victim = None
        excepted_url = ""
        self.assertEqual(self.killmail.get_image_url(), excepted_url)

    def test_get_victim_name(self):
        expected_url = "Gneuten"
        self.assertEqual(self.killmail.get_or_unknown_victim_name(), expected_url)

        self.killmail.victim = None
        expected_url = "Unknown"
        self.assertEqual(self.killmail.get_or_unknown_victim_name(), expected_url)

    def test_get_victim_ship_name(self):
        expected_url = "Victim Ship I"
        self.assertEqual(self.killmail.get_or_unknown_victim_ship_name(), expected_url)

        self.killmail.victim_ship = None
        expected_url = "Unknown"

    def test_evaluate_victim_id(self):
        expected_url = 1001
        self.assertEqual(self.killmail.evaluate_victim_id(), expected_url)

        self.killmail.victim = None
        expected_url = 20000001
        self.assertEqual(self.killmail.evaluate_victim_id(), expected_url)

        self.killmail.victim_corporation_id = None
        expected_url = 30000001
        self.assertEqual(self.killmail.evaluate_victim_id(), expected_url)

        self.killmail.victim_alliance_id = None
        expected_url = 0
        self.assertEqual(self.killmail.evaluate_victim_id(), expected_url)

    def test_evaluate_zkb_link(self):
        expected_url = "https://zkillboard.com/character/1001/"
        self.assertEqual(self.killmail.evaluate_zkb_link(), expected_url)

        self.killmail.victim.category = "corporation"
        expected_url = "https://zkillboard.com/corporation/20000001/"
        self.assertEqual(self.killmail.evaluate_zkb_link(), expected_url)

        self.killmail.victim.category = "alliance"
        expected_url = "https://zkillboard.com/alliance/30000001/"
        self.assertEqual(self.killmail.evaluate_zkb_link(), expected_url)

    def test_evaluate_portrait(self):
        expected_url = "https://images.evetech.net/characters/1001/portrait?size=256"
        self.assertEqual(self.killmail.evaluate_portrait(), expected_url)

        self.killmail.victim.category = "corporation"
        expected_url = "https://images.evetech.net/corporations/20000001/logo?size=256"
        self.assertEqual(self.killmail.evaluate_portrait(), expected_url)

        self.killmail.victim.category = "alliance"
        expected_url = "https://images.evetech.net/alliances/30000001/logo?size=256"
        self.assertEqual(self.killmail.evaluate_portrait(), expected_url)

    def test_get_ship_image_url(self):
        expected_url = "https://images.evetech.net/types/10001/render?size=32"
        self.assertEqual(self.killmail.get_ship_image_url(), expected_url)

        self.killmail.victim_ship = None
        expected_url = ""
        self.assertEqual(self.killmail.get_ship_image_url(), expected_url)



from .admin import KillboardAdminApiEndpoints
from .killboard import KillboardApiEndpoints
from .stats import KillboardStatsApiEndpoints


def setup(api):
    KillboardApiEndpoints(api)
    KillboardStatsApiEndpoints(api)
    KillboardAdminApiEndpoints(api)



import hashlib

from django.core.cache import cache
from django.db.models import Q, Sum
from eveuniverse.models import EveEntity

from killstats.api.account_manager import AccountManager
from killstats.api.helpers import get_alliances, get_corporations
from killstats.api.killboard_manager import (
    format_killmail,
    format_killmail_details,
    killboard_hall,
)
from killstats.app_settings import KILLBOARD_API_CACHE_LIFETIME
from killstats.hooks import get_extension_logger
from killstats.models.killboard import Killmail
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit

logger = get_extension_logger(__name__)

KILLMAIL_MAPPING = {
    0: "killmail_id",
    1: "victim_ship__name",
    2: "victim",
    3: "victim__name",
    4: "victim_total_value",
    5: "killmail_date",
}


def set_cache_key(cache_key, output, timeout=KILLBOARD_API_CACHE_LIFETIME):
    if not cache_key:
        return False

    logger.debug("Cache Set: %s", cache_key)
    cache.set(
        key=cache_key,
        value=output,
        timeout=60 * timeout,
    )
    return True


def cache_sytem(request, cache_name, entity_id) -> tuple:
    cache_id = get_unique_id(request, entity_id)

    cache_key = f"{cache_name}_{cache_id}"
    output = cache.get(cache_key)

    if output is not None:
        logger.debug("Cache hit: %s", cache_key)
        return output, None
    return None, cache_key


def get_unique_id(request, entity_id) -> str:
    if entity_id != 0:
        return entity_id

    corp = get_corporations(request)
    ally = get_alliances(request)
    corp_ids = CorporationsAudit.objects.filter(
        corporation__corporation_id__in=corp
    ).values_list("corporation__corporation_id", flat=True)
    ally_ids = AlliancesAudit.objects.filter(
        alliance__alliance_id__in=ally
    ).values_list("alliance__alliance_id", flat=True)
    entities = list(corp_ids) + list(ally_ids)

    # Combine all IDs into a single string
    combined_ids = "_".join(map(str, sorted(entities)))

    # Create a unique ID from the combined IDs
    unique_id = hashlib.md5(combined_ids.encode()).hexdigest()
    return unique_id


def get_entities(request, entity_type: str, entity_id: int):
    if entity_id == 0:
        if entity_type == "alliance":
            entities = get_alliances(request)
        else:
            entities = get_corporations(request)
    else:
        entities = [entity_id]
    return entities


# pylint: disable=too-many-locals, too-many-positional-arguments
def get_killmails_data(request, month, year, entity_type: str, entity_id: int, mode):
    entities = get_entities(request, entity_type, entity_id)

    # Datatables parameters
    start = int(request.GET.get("start", 0))
    length = int(request.GET.get("length", 25))
    limit = start + length

    search_value = request.GET.get("search[value]", "")
    order_column_index = int(request.GET.get("order[0][column]", 0))
    order_dir = request.GET.get("order[0][dir]", "desc")
    order_column = KILLMAIL_MAPPING.get(order_column_index, "killmail_date")
    order_by = f"{'-' if order_dir == 'desc' else ''}{order_column}"

    killmails = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by(order_by)
    ).filter_entities(entities)

    if mode == "losses":
        killmails = killmails.filter_entities_losses(entities)
    else:
        killmails = killmails.filter_entities_kills(entities)

    totalvalue = (
        killmails.aggregate(totalvalue=Sum("victim_total_value"))["totalvalue"],
    )

    if search_value:
        killmails = killmails.filter(
            Q(victim__name__icontains=search_value)
            | Q(victim_ship__name__icontains=search_value)
            | Q(victim_total_value__icontains=search_value)
            | Q(killmail_date__icontains=search_value)
        )

    total_count = killmails.count()
    record_count = killmails.count()
    killmails = killmails[start:limit]

    output = []
    for killmail in killmails:
        output.append(
            {
                "killmail_id": killmail.killmail_id,
                "killmail_date": killmail.killmail_date,
                "victim": {
                    "id": killmail.victim_id,
                    "name": killmail.victim.name if killmail.victim else "Unknown",
                },
                "victim_ship": {
                    "id": killmail.victim_ship_id,
                    "name": (
                        killmail.victim_ship.name if killmail.victim_ship else "Unknown"
                    ),
                },
                "victim_corporation_id": killmail.victim_corporation_id,
                "victim_alliance_id": killmail.victim_alliance_id,
                "hash": killmail.hash,
                "victim_total_value": killmail.victim_total_value,
                "victim_fitted_value": killmail.victim_fitted_value,
                "victim_destroyed_value": killmail.victim_destroyed_value,
                "victim_dropped_value": killmail.victim_dropped_value,
                "victim_region_id": killmail.victim_region_id,
                "victim_solar_system_id": killmail.victim_solar_system_id,
                "victim_position_x": killmail.victim_position_x,
                "victim_position_y": killmail.victim_position_y,
                "victim_position_z": killmail.victim_position_z,
            }
        )

    return {
        "draw": int(request.GET.get("draw", 1)),
        "recordsTotal": total_count,
        "recordsFiltered": total_count if not search_value else record_count,
        "data": output,
        "totalvalue": totalvalue,
    }


def get_killstats_halls(request, month, year, entity_type: str, entity_id: int):
    entities = get_entities(request, entity_type, entity_id)
    account = AccountManager(entities=entities)
    mains, _ = account.get_mains_alts()

    killmails = (
        Killmail.objects.prefetch_related("victim", "victim_ship").filter(
            killmail_date__year=year,
            killmail_date__month=month,
        )
    ).filter_entities(entities)

    shame, fame = killboard_hall(killmails, entities, mains)

    halls = []
    halls.append(
        {
            "shame": shame,
            "fame": fame,
        }
    )

    return halls


def get_top_victim(request, month, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    top_victim_querry = killmail_year.get_top_victim_stats(entities)

    if not top_victim_querry:
        return {}

    top_victim_dict = format_killmail_details(
        top_victim_querry,
        title="Top Victim:",
        count=top_victim_querry.kill_count,
        stats_type="character",
    )

    return top_victim_dict


def get_top_killer(request, month, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    top_killer_querry = killmail_year.get_top_killer_stats(entities)

    if not top_killer_querry:
        return {}

    top_killer_dict = format_killmail_details(
        top_killer_querry,
        title="Top Killer:",
        count=top_killer_querry.kill_count,
        stats_type="character",
    )

    return top_killer_dict


def get_alltime_victim(request, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    alltime_victim_querry = killmail_year.get_top_victim_stats(entities)

    if not alltime_victim_querry:
        return {}

    alltime_victim_dict = format_killmail_details(
        alltime_victim_querry,
        title="Alltime Victim:",
        count=alltime_victim_querry.kill_count,
        stats_type="character",
    )

    return alltime_victim_dict


def get_alltime_killer(request, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    alltime_killer_querry = killmail_year.get_top_killer_stats(entities)

    if not alltime_killer_querry:
        return {}

    alltime_killer_dict = format_killmail_details(
        alltime_killer_querry,
        title="Alltime Killer:",
        count=alltime_killer_querry.kill_count,
        stats_type="character",
    )

    return alltime_killer_dict


def get_worst_ship(request, month, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    worst_ship_querry = killmail_year.get_worst_ship_stats(entities)

    if not worst_ship_querry:
        return {}

    worst_ship_dict = format_killmail_details(
        worst_ship_querry,
        loss=True,
        title="Worst Ship:",
        count=worst_ship_querry.ship_count,
        stats_type="ship",
    )

    return worst_ship_dict


def get_top_ship(request, month, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    top_ship_querry = killmail_year.get_top_ship_stats(entities)

    if not top_ship_querry:
        return {}

    top_ship_dict = format_killmail_details(
        top_ship_querry,
        title="Top Ship:",
        count=top_ship_querry.ship_count,
        stats_type="ship",
    )

    return top_ship_dict


def get_top_kill(request, month, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    top_kill_querry = killmail_year.get_highest_kill_stats(entities)

    if not top_kill_querry:
        return {}

    top_kill_dict = format_killmail(
        top_kill_querry,
        title="Top Kill:",
    )

    return top_kill_dict


def get_top_loss(request, month, year, entity_type: str, entity_id: int) -> dict:
    entities = get_entities(request, entity_type, entity_id)

    killmail_year = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    top_loss_querry = killmail_year.get_highest_loss_stats(entities)

    if not top_loss_querry:
        return {}

    top_loss_dict = format_killmail(
        top_loss_querry,
        title="Top Loss:",
    )

    return top_loss_dict


def get_top_10(request, month, year, entity_type: str, entity_id: int) -> list:
    entities = get_entities(request, entity_type, entity_id)
    account = AccountManager(entities=entities)
    mains_dict, _ = account.get_mains_alts()

    killmail = (
        Killmail.objects.prefetch_related("victim", "victim_ship")
        .filter(killmail_date__year=year, killmail_date__month=month)
        .order_by("-killmail_date")
    ).filter_entities(entities)

    km_ids = killmail.values_list("killmail_id", flat=True)

    top_10_querry = killmail.get_top_10_killers(entities, km_ids)

    if not top_10_querry:
        return {}

    # Convert QuerySet to list
    top_10_list = list(top_10_querry)

    # Add character_name to each entry
    for entry in top_10_list:
        character_id = entry["character_id"]
        try:
            character = EveEntity.objects.get(id=character_id)
            entry["character_name"] = character.name

            # Check if the character is an alt and add the main character's name
            for main_id, data in mains_dict.items():
                alts = [alt.character_id for alt in data["alts"]]
                if character_id in alts and character_id != main_id:
                    main_character = data["main"]
                    entry["character_name"] += f" ({main_character.character_name})"
                    break
        except EveEntity.DoesNotExist:
            entry["character_name"] = "Unknown"
        except AttributeError:
            entry["character_name"] = "Unknown"

    return top_10_list
